/* Copyright (c) 2001, Stanford University
	All rights reserved.

	See the file LICENSE.txt for information on redistributing this software. */
	

/* DO NOT EDIT - THIS FILE GENERATED BY THE glloader.py SCRIPT */
#include "cr_error.h"
#include "cr_dll.h"
#include "cr_spu.h"
#include "cr_string.h"
#include "cr_error.h"
#include "cr_environment.h"

#include <stdio.h>
#if defined(WINDOWS)
#include <windows.h>
#include <process.h>
#include <direct.h>
#define SYSTEM_GL "opengl32.dll"
#elif defined (DARWIN)
#define SYSTEM_GL "libGL.dylib"
#define SYSTEM_CGL "OpenGL"
# ifndef VBOX_WITH_COCOA_QT
#  define SYSTEM_AGL "AGL"
# endif
#include <string.h> /* VBOX */
#elif defined(IRIX) || defined(IRIX64) || defined(Linux) || defined(FreeBSD) || defined(AIX) || defined(SunOS) || defined(OSF1)
#include <string.h>
#if defined(AIX)
#define SYSTEM_GL "libGL.o"
#else
#define SYSTEM_GL "libGL.so.1"
#endif
typedef void (*glxfuncptr)();
extern glxfuncptr glxGetProcAddressARB( const GLubyte *name );
#else
#error I don't know where your system's GL lives.  Too bad.
#endif

static CRDLL *glDll = NULL;

#ifdef DARWIN
#define SYSTEM_GL_LIB_DIR   "/System/Library/Frameworks/OpenGL.framework/Libraries"
#define SYSTEM_CGL_DIR  "/System/Library/Frameworks/OpenGL.framework"
static CRDLL *cglDll = NULL;
# ifndef VBOX_WITH_COCOA_QT
#  define SYSTEM_AGL_DIR  "/System/Library/Frameworks/AGL.framework"
static CRDLL *aglDll = NULL;
# endif
#endif

#if defined(WINDOWS)
#define GLLOADER_APIENTRY __stdcall
#else
#define GLLOADER_APIENTRY
#endif

#if defined(WINDOWS) && (defined(DEBUG_leo) || defined(DEBUG_ll158262) || defined(DEBUG_misha))
# define CR_NO_GL_SYSTEM_PATH 1
#endif

/*
 * Add an entry to the SPUNamedFunctionTable
 */
static int
fillin( SPUNamedFunctionTable *entry, const char *funcName, SPUGenericFunction funcPtr )
{
	if (funcPtr) {
		entry->name = crStrdup( funcName );
		entry->fn = funcPtr;
		return 1;
	}
	return 0;
}

#ifndef WINDOWS
static int FileExists(char *directory, char *filename)
{
	FILE *f;
	char fullFilename[8096];

	crStrcpy(fullFilename, directory);
	crStrcat(fullFilename, "/");
	crStrcat(fullFilename, filename);

	f = fopen(fullFilename, "r");
	if (f) {
		fclose(f);
		return 1;
	}
	else {
	    return 0;
	}
}
#endif


/*
 * Locate the native OpenGL library, open it and return shared library
 * handle.
 */
static CRDLL *
__findSystemLib( const char *provided_system_path, char *lib )
{
	CRDLL *dll;
	char system_path[8096];

	memset(system_path, 0, sizeof(system_path));

	if (provided_system_path && (crStrlen(provided_system_path) > 0) )
	{
		crStrcpy( system_path, provided_system_path );
	}
	else
	{
#if defined(WINDOWS)
		GetSystemDirectory(system_path, MAX_PATH);
#elif defined(IRIX) || defined(IRIX64)
#ifdef IRIX_64BIT
		crStrcpy( system_path, "/usr/lib64" );
#else
		crStrcpy( system_path, "/usr/lib32" );
#endif
#elif defined(PLAYSTATION2)
		crStrcpy( system_path, "/usr/X11R6/lib" );
#else
		/* On RedHat 9, the correct default system directory
		 * is /usr/lib/tls/ (and if /usr/lib/ is used,
		 * the dynamic loader will generate a floating point
		 * exception SIGFPE).  On other systems, including
		 * earlier versions of RedHat, the OpenGL library
		 * lives in /usr/lib.   We'll use the /usr/lib/tls/
		 * version if it exists; otherwise, we'll use /usr/lib.
		 */
		/*crStrcpy(system_path, "/usr/lib");*/
#if defined(__linux__) && defined(__amd64__)
		/*if (sizeof(void *) == 8 && FileExists("/usr/lib64", lib)) {
			crStrcat(system_path, "64");
		}*/
#endif
		/*if (FileExists("/usr/lib/tls", lib) ||
		    FileExists("/usr/lib64/tls", lib)) {
			crStrcat(system_path, "/tls");
		}*/
#endif
	}
#if !defined(__linux__) && !defined(SunOS) && !defined(__FreeBSD__)
	crStrcat( system_path, "/" );
#endif
#if !defined(CR_NO_GL_SYSTEM_PATH)
	crStrcat( system_path, lib );
	dll = crDLLOpen(system_path, 1 /*resolveGlobal*/);
#else
	dll = crDLLOpen(lib, 1 /*resolveGlobal*/);
#endif
	return dll;
}


static CRDLL *
#ifdef DARWIN
__findSystemGL( const char *provided_system_path, const char *default_system_path, char *provided_lib_name )
#else
__findSystemGL( const char *provided_system_path )
#endif
{
#ifdef DARWIN
	const char *the_path = (provided_system_path && crStrlen(provided_system_path) > 0) ? provided_system_path : default_system_path;

	/* Fallback for loading frameworks */
	if( !provided_lib_name )
		return crDLLOpen( the_path, 1 );
	else
		return __findSystemLib( the_path, provided_lib_name );
#else
	return __findSystemLib( provided_system_path, SYSTEM_GL );
#endif
}

static SPUGenericFunction
findExtFunction( const crOpenGLInterface *interface, const char *funcName )
{
#ifdef WINDOWS
	if (interface->wglGetProcAddress)
		return (SPUGenericFunction) interface->wglGetProcAddress( funcName );
	else
		return (SPUGenericFunction) NULL;
#else
	/* XXX for some reason, the NVIDIA glXGetProcAddressARB() function
	 * returns pointers that cause Chromium to crash.  If we use the
	 * pointer returned by crDLLGetNoError() instead, we're OK.
	 */
	SPUGenericFunction f = crDLLGetNoError(glDll, funcName);
	if (f)
		return f;
#if !defined(DARWIN)
	else if (interface->glXGetProcAddressARB)
		return interface->glXGetProcAddressARB( (const GLubyte *) funcName );
#endif
	else
		return NULL;
#endif
}

static void GLLOADER_APIENTRY NopActiveStencilFaceEXT(GLenum face)
{
	(void) face;
}

static void GLLOADER_APIENTRY NopActiveTextureARB(GLenum texture)
{
	(void) texture;
}

static GLboolean GLLOADER_APIENTRY NopAreProgramsResidentNV(GLsizei n, const GLuint * ids, GLboolean * residences)
{
	(void) n;
	(void) ids;
	(void) residences;
	return 0;
}

static void GLLOADER_APIENTRY NopAttachShader(GLuint program, GLuint shader)
{
	(void) program;
	(void) shader;
}

static void GLLOADER_APIENTRY NopBarrierCreateCR(GLuint name, GLuint count)
{
	(void) name;
	(void) count;
}

static void GLLOADER_APIENTRY NopBarrierDestroyCR(GLuint name)
{
	(void) name;
}

static void GLLOADER_APIENTRY NopBarrierExecCR(GLuint name)
{
	(void) name;
}

static void GLLOADER_APIENTRY NopBeginQueryARB(GLenum target, GLuint id)
{
	(void) target;
	(void) id;
}

static void GLLOADER_APIENTRY NopBindAttribLocation(GLuint program, GLuint index, const char * name)
{
	(void) program;
	(void) index;
	(void) name;
}

static void GLLOADER_APIENTRY NopBindBufferARB(GLenum target, GLuint buffer)
{
	(void) target;
	(void) buffer;
}

static void GLLOADER_APIENTRY NopBindFramebufferEXT(GLenum target, GLuint framebuffer)
{
	(void) target;
	(void) framebuffer;
}

static void GLLOADER_APIENTRY NopBindProgramARB(GLenum target, GLuint program)
{
	(void) target;
	(void) program;
}

static void GLLOADER_APIENTRY NopBindProgramNV(GLenum target, GLuint id)
{
	(void) target;
	(void) id;
}

static void GLLOADER_APIENTRY NopBindRenderbufferEXT(GLenum target, GLuint renderbuffer)
{
	(void) target;
	(void) renderbuffer;
}

static void GLLOADER_APIENTRY NopBlendColorEXT(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
{
	(void) red;
	(void) green;
	(void) blue;
	(void) alpha;
}

static void GLLOADER_APIENTRY NopBlendEquationEXT(GLenum mode)
{
	(void) mode;
}

static void GLLOADER_APIENTRY NopBlendEquationSeparate(GLenum modeRGB, GLenum modeAlpha)
{
	(void) modeRGB;
	(void) modeAlpha;
}

static void GLLOADER_APIENTRY NopBlendFuncSeparateEXT(GLenum sfactorRGB, GLenum dfactorRGB, GLenum sfactorA, GLenum dfactorA)
{
	(void) sfactorRGB;
	(void) dfactorRGB;
	(void) sfactorA;
	(void) dfactorA;
}

static void GLLOADER_APIENTRY NopBlitFramebufferEXT(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1, GLint dstX0, GLint dstY0, GLint dstX1, GLint dstY1, GLbitfield mask, GLenum filter)
{
	(void) srcX0;
	(void) srcY0;
	(void) srcX1;
	(void) srcY1;
	(void) dstX0;
	(void) dstY0;
	(void) dstX1;
	(void) dstY1;
	(void) mask;
	(void) filter;
}

static void GLLOADER_APIENTRY NopBoundsInfoCR(const CRrecti * bounds, const GLbyte * payload, GLint len, GLint num_opcodes)
{
	(void) bounds;
	(void) payload;
	(void) len;
	(void) num_opcodes;
}

static void GLLOADER_APIENTRY NopBufferDataARB(GLenum target, GLsizeiptrARB size, const GLvoid * data, GLenum usage)
{
	(void) target;
	(void) size;
	(void) data;
	(void) usage;
}

static void GLLOADER_APIENTRY NopBufferSubDataARB(GLenum target, GLintptrARB offset, GLsizeiptrARB size, const GLvoid * data)
{
	(void) target;
	(void) offset;
	(void) size;
	(void) data;
}

static GLenum GLLOADER_APIENTRY NopCheckFramebufferStatusEXT(GLenum target)
{
	(void) target;
	return 0;
}

static void GLLOADER_APIENTRY NopChromiumParameterfCR(GLenum target, GLfloat value)
{
	(void) target;
	(void) value;
}

static void GLLOADER_APIENTRY NopChromiumParameteriCR(GLenum target, GLint value)
{
	(void) target;
	(void) value;
}

static void GLLOADER_APIENTRY NopChromiumParametervCR(GLenum target, GLenum type, GLsizei count, const GLvoid * values)
{
	(void) target;
	(void) type;
	(void) count;
	(void) values;
}

static void GLLOADER_APIENTRY NopClientActiveTextureARB(GLenum texture)
{
	(void) texture;
}

static void GLLOADER_APIENTRY NopCombinerInputNV(GLenum stage, GLenum portion, GLenum variable, GLenum input, GLenum mapping, GLenum componentUsage)
{
	(void) stage;
	(void) portion;
	(void) variable;
	(void) input;
	(void) mapping;
	(void) componentUsage;
}

static void GLLOADER_APIENTRY NopCombinerOutputNV(GLenum stage, GLenum portion, GLenum abOutput, GLenum cdOutput, GLenum sumOutput, GLenum scale, GLenum bias, GLboolean abDotProduct, GLboolean cdDotProduct, GLboolean muxSum)
{
	(void) stage;
	(void) portion;
	(void) abOutput;
	(void) cdOutput;
	(void) sumOutput;
	(void) scale;
	(void) bias;
	(void) abDotProduct;
	(void) cdDotProduct;
	(void) muxSum;
}

static void GLLOADER_APIENTRY NopCombinerParameterfNV(GLenum pname, GLfloat param)
{
	(void) pname;
	(void) param;
}

static void GLLOADER_APIENTRY NopCombinerParameterfvNV(GLenum pname, const GLfloat * params)
{
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopCombinerParameteriNV(GLenum pname, GLint param)
{
	(void) pname;
	(void) param;
}

static void GLLOADER_APIENTRY NopCombinerParameterivNV(GLenum pname, const GLint * params)
{
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopCombinerStageParameterfvNV(GLenum stage, GLenum pname, const GLfloat * params)
{
	(void) stage;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopCompileShader(GLuint shader)
{
	(void) shader;
}

static void GLLOADER_APIENTRY NopCompressedTexImage1DARB(GLenum target, GLint level, GLenum internalFormat, GLsizei width, GLint border, GLsizei imagesize, const GLvoid * data)
{
	(void) target;
	(void) level;
	(void) internalFormat;
	(void) width;
	(void) border;
	(void) imagesize;
	(void) data;
}

static void GLLOADER_APIENTRY NopCompressedTexImage2DARB(GLenum target, GLint level, GLenum internalFormat, GLsizei width, GLsizei height, GLint border, GLsizei imagesize, const GLvoid * data)
{
	(void) target;
	(void) level;
	(void) internalFormat;
	(void) width;
	(void) height;
	(void) border;
	(void) imagesize;
	(void) data;
}

static void GLLOADER_APIENTRY NopCompressedTexImage3DARB(GLenum target, GLint level, GLenum internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLsizei imagesize, const GLvoid * data)
{
	(void) target;
	(void) level;
	(void) internalFormat;
	(void) width;
	(void) height;
	(void) depth;
	(void) border;
	(void) imagesize;
	(void) data;
}

static void GLLOADER_APIENTRY NopCompressedTexSubImage1DARB(GLenum target, GLint level, GLint xoffset, GLsizei width, GLenum format, GLsizei imagesize, const GLvoid * data)
{
	(void) target;
	(void) level;
	(void) xoffset;
	(void) width;
	(void) format;
	(void) imagesize;
	(void) data;
}

static void GLLOADER_APIENTRY NopCompressedTexSubImage2DARB(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLsizei width, GLsizei height, GLenum format, GLsizei imagesize, const GLvoid * data)
{
	(void) target;
	(void) level;
	(void) xoffset;
	(void) yoffset;
	(void) width;
	(void) height;
	(void) format;
	(void) imagesize;
	(void) data;
}

static void GLLOADER_APIENTRY NopCompressedTexSubImage3DARB(GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint zoffset, GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLsizei imagesize, const GLvoid * data)
{
	(void) target;
	(void) level;
	(void) xoffset;
	(void) yoffset;
	(void) zoffset;
	(void) width;
	(void) height;
	(void) depth;
	(void) format;
	(void) imagesize;
	(void) data;
}

static GLint GLLOADER_APIENTRY NopCreateContext(const char * dpyName, GLint visual, GLint shareCtx)
{
	(void) dpyName;
	(void) visual;
	(void) shareCtx;
	return 0;
}

static GLuint GLLOADER_APIENTRY NopCreateProgram(void)
{
	return 0;
}

static GLuint GLLOADER_APIENTRY NopCreateShader(GLenum type)
{
	(void) type;
	return 0;
}

static void GLLOADER_APIENTRY NopDeleteBuffersARB(GLsizei n, const GLuint * buffer)
{
	(void) n;
	(void) buffer;
}

static void GLLOADER_APIENTRY NopDeleteFencesNV(GLsizei n, const GLuint * fences)
{
	(void) n;
	(void) fences;
}

static void GLLOADER_APIENTRY NopDeleteFramebuffersEXT(GLsizei n, const GLuint * framebuffers)
{
	(void) n;
	(void) framebuffers;
}

static void GLLOADER_APIENTRY NopDeleteObjectARB(GLhandleARB obj)
{
	(void) obj;
}

static void GLLOADER_APIENTRY NopDeleteProgram(GLuint program)
{
	(void) program;
}

static void GLLOADER_APIENTRY NopDeleteProgramsARB(GLsizei n, const GLuint * programs)
{
	(void) n;
	(void) programs;
}

static void GLLOADER_APIENTRY NopDeleteQueriesARB(GLsizei n, const GLuint * ids)
{
	(void) n;
	(void) ids;
}

static void GLLOADER_APIENTRY NopDeleteRenderbuffersEXT(GLsizei n, const GLuint * renderbuffers)
{
	(void) n;
	(void) renderbuffers;
}

static void GLLOADER_APIENTRY NopDeleteShader(GLuint shader)
{
	(void) shader;
}

static void GLLOADER_APIENTRY NopDestroyContext(GLint ctx)
{
	(void) ctx;
}

static void GLLOADER_APIENTRY NopDetachShader(GLuint program, GLuint shader)
{
	(void) program;
	(void) shader;
}

static void GLLOADER_APIENTRY NopDisableVertexAttribArrayARB(GLuint index)
{
	(void) index;
}

static void GLLOADER_APIENTRY NopDrawBuffers(GLsizei n, const GLenum* bufs)
{
	(void) n;
	(void) bufs;
}

static void GLLOADER_APIENTRY NopEnableVertexAttribArrayARB(GLuint index)
{
	(void) index;
}

static void GLLOADER_APIENTRY NopEndQueryARB(GLenum target)
{
	(void) target;
}

static void GLLOADER_APIENTRY NopExecuteProgramNV(GLenum target, GLuint id, const GLfloat * params)
{
	(void) target;
	(void) id;
	(void) params;
}

static void GLLOADER_APIENTRY NopFinalCombinerInputNV(GLenum variable, GLenum input, GLenum mapping, GLenum componentUsage)
{
	(void) variable;
	(void) input;
	(void) mapping;
	(void) componentUsage;
}

static void GLLOADER_APIENTRY NopFinishFenceNV(GLuint fence)
{
	(void) fence;
}

static void GLLOADER_APIENTRY NopFlushVertexArrayRangeNV(void)
{
}

static void GLLOADER_APIENTRY NopFogCoordPointerEXT(GLenum type, GLsizei stride, const GLvoid * pointer)
{
	(void) type;
	(void) stride;
	(void) pointer;
}

static void GLLOADER_APIENTRY NopFogCoorddEXT(GLdouble coord)
{
	(void) coord;
}

static void GLLOADER_APIENTRY NopFogCoorddvEXT(const GLdouble * coord)
{
	(void) coord;
}

static void GLLOADER_APIENTRY NopFogCoordfEXT(GLfloat coord)
{
	(void) coord;
}

static void GLLOADER_APIENTRY NopFogCoordfvEXT(const GLfloat * coord)
{
	(void) coord;
}

static void GLLOADER_APIENTRY NopFramebufferRenderbufferEXT(GLenum target, GLenum attachment, GLenum renderbuffertarget, GLuint renderbuffer)
{
	(void) target;
	(void) attachment;
	(void) renderbuffertarget;
	(void) renderbuffer;
}

static void GLLOADER_APIENTRY NopFramebufferTexture1DEXT(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level)
{
	(void) target;
	(void) attachment;
	(void) textarget;
	(void) texture;
	(void) level;
}

static void GLLOADER_APIENTRY NopFramebufferTexture2DEXT(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level)
{
	(void) target;
	(void) attachment;
	(void) textarget;
	(void) texture;
	(void) level;
}

static void GLLOADER_APIENTRY NopFramebufferTexture3DEXT(GLenum target, GLenum attachment, GLenum textarget, GLuint texture, GLint level, GLint zoffset)
{
	(void) target;
	(void) attachment;
	(void) textarget;
	(void) texture;
	(void) level;
	(void) zoffset;
}

static void GLLOADER_APIENTRY NopGenBuffersARB(GLsizei n, GLuint * buffer)
{
	(void) n;
	(void) buffer;
}

static void GLLOADER_APIENTRY NopGenFencesNV(GLsizei n, GLuint * fences)
{
	(void) n;
	(void) fences;
}

static void GLLOADER_APIENTRY NopGenFramebuffersEXT(GLsizei n, GLuint * framebuffers)
{
	(void) n;
	(void) framebuffers;
}

static void GLLOADER_APIENTRY NopGenProgramsARB(GLsizei n, GLuint * programs)
{
	(void) n;
	(void) programs;
}

static void GLLOADER_APIENTRY NopGenProgramsNV(GLsizei n, GLuint * ids)
{
	(void) n;
	(void) ids;
}

static void GLLOADER_APIENTRY NopGenQueriesARB(GLsizei n, GLuint * ids)
{
	(void) n;
	(void) ids;
}

static void GLLOADER_APIENTRY NopGenRenderbuffersEXT(GLsizei n, GLuint * renderbuffers)
{
	(void) n;
	(void) renderbuffers;
}

static void GLLOADER_APIENTRY NopGenerateMipmapEXT(GLenum target)
{
	(void) target;
}

static void GLLOADER_APIENTRY NopGetActiveAttrib(GLuint program, GLuint index, GLsizei bufSize, GLsizei * length, GLint * size, GLenum * type, char * name)
{
	(void) program;
	(void) index;
	(void) bufSize;
	(void) length;
	(void) size;
	(void) type;
	(void) name;
}

static void GLLOADER_APIENTRY NopGetActiveUniform(GLuint program, GLuint index, GLsizei bufSize, GLsizei * length, GLint * size, GLenum * type, char * name)
{
	(void) program;
	(void) index;
	(void) bufSize;
	(void) length;
	(void) size;
	(void) type;
	(void) name;
}

static void GLLOADER_APIENTRY NopGetAttachedObjectsARB(GLhandleARB containerObj, GLsizei maxCount, GLsizei * count, GLhandleARB * obj)
{
	(void) containerObj;
	(void) maxCount;
	(void) count;
	(void) obj;
}

static void GLLOADER_APIENTRY NopGetAttachedShaders(GLuint program, GLsizei maxCount, GLsizei * count, GLuint * shaders)
{
	(void) program;
	(void) maxCount;
	(void) count;
	(void) shaders;
}

static GLint GLLOADER_APIENTRY NopGetAttribLocation(GLuint program, const char * name)
{
	(void) program;
	(void) name;
	return 0;
}

static void GLLOADER_APIENTRY NopGetBufferParameterivARB(GLenum target, GLenum pname, GLint * params)
{
	(void) target;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetBufferPointervARB(GLenum target, GLenum pname, GLvoid ** params)
{
	(void) target;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetBufferSubDataARB(GLenum target, GLintptrARB offset, GLsizeiptrARB size, void * data)
{
	(void) target;
	(void) offset;
	(void) size;
	(void) data;
}

static void GLLOADER_APIENTRY NopGetChromiumParametervCR(GLenum target, GLuint index, GLenum type, GLsizei count, GLvoid * values)
{
	(void) target;
	(void) index;
	(void) type;
	(void) count;
	(void) values;
}

static void GLLOADER_APIENTRY NopGetCombinerInputParameterfvNV(GLenum stage, GLenum portion, GLenum variable, GLenum pname, GLfloat * params)
{
	(void) stage;
	(void) portion;
	(void) variable;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetCombinerInputParameterivNV(GLenum stage, GLenum portion, GLenum variable, GLenum pname, GLint * params)
{
	(void) stage;
	(void) portion;
	(void) variable;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetCombinerOutputParameterfvNV(GLenum stage, GLenum portion, GLenum pname, GLfloat * params)
{
	(void) stage;
	(void) portion;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetCombinerOutputParameterivNV(GLenum stage, GLenum portion, GLenum pname, GLint * params)
{
	(void) stage;
	(void) portion;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetCombinerStageParameterfvNV(GLenum stage, GLenum pname, GLfloat * params)
{
	(void) stage;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetCompressedTexImageARB(GLenum target, GLint level, GLvoid * img)
{
	(void) target;
	(void) level;
	(void) img;
}

static void GLLOADER_APIENTRY NopGetFenceivNV(GLuint fence, GLenum pname, GLint * params)
{
	(void) fence;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetFinalCombinerInputParameterfvNV(GLenum variable, GLenum pname, GLfloat * params)
{
	(void) variable;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetFinalCombinerInputParameterivNV(GLenum variable, GLenum pname, GLint * params)
{
	(void) variable;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetFramebufferAttachmentParameterivEXT(GLenum target, GLenum attachment, GLenum pname, GLint * params)
{
	(void) target;
	(void) attachment;
	(void) pname;
	(void) params;
}

static GLhandleARB GLLOADER_APIENTRY NopGetHandleARB(GLenum pname)
{
	(void) pname;
	return 0;
}

static void GLLOADER_APIENTRY NopGetInfoLogARB(GLhandleARB obj, GLsizei maxLength, GLsizei * length, GLcharARB * infoLog)
{
	(void) obj;
	(void) maxLength;
	(void) length;
	(void) infoLog;
}

static void GLLOADER_APIENTRY NopGetObjectParameterfvARB(GLhandleARB obj, GLenum pname, GLfloat * params)
{
	(void) obj;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetObjectParameterivARB(GLhandleARB obj, GLenum pname, GLint * params)
{
	(void) obj;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetProgramEnvParameterdvARB(GLenum target, GLuint index, GLdouble * params)
{
	(void) target;
	(void) index;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetProgramEnvParameterfvARB(GLenum target, GLuint index, GLfloat * params)
{
	(void) target;
	(void) index;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetProgramInfoLog(GLuint program, GLsizei bufSize, GLsizei * length, char * infoLog)
{
	(void) program;
	(void) bufSize;
	(void) length;
	(void) infoLog;
}

static void GLLOADER_APIENTRY NopGetProgramLocalParameterdvARB(GLenum target, GLuint index, GLdouble * params)
{
	(void) target;
	(void) index;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetProgramLocalParameterfvARB(GLenum target, GLuint index, GLfloat * params)
{
	(void) target;
	(void) index;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetProgramNamedParameterdvNV(GLuint id, GLsizei len, const GLubyte * name, GLdouble * params)
{
	(void) id;
	(void) len;
	(void) name;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetProgramNamedParameterfvNV(GLuint id, GLsizei len, const GLubyte * name, GLfloat * params)
{
	(void) id;
	(void) len;
	(void) name;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetProgramParameterdvNV(GLenum target, GLuint index, GLenum pname, GLdouble * params)
{
	(void) target;
	(void) index;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetProgramParameterfvNV(GLenum target, GLuint index, GLenum pname, GLfloat * params)
{
	(void) target;
	(void) index;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetProgramStringARB(GLenum target, GLenum pname, GLvoid * string)
{
	(void) target;
	(void) pname;
	(void) string;
}

static void GLLOADER_APIENTRY NopGetProgramStringNV(GLuint id, GLenum pname, GLubyte * program)
{
	(void) id;
	(void) pname;
	(void) program;
}

static void GLLOADER_APIENTRY NopGetProgramiv(GLuint program, GLenum pname, GLint * params)
{
	(void) program;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetProgramivARB(GLenum target, GLenum pname, GLint * params)
{
	(void) target;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetProgramivNV(GLuint id, GLenum pname, GLint * params)
{
	(void) id;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetQueryObjectivARB(GLuint id, GLenum pname, GLint * params)
{
	(void) id;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetQueryObjectuivARB(GLuint id, GLenum pname, GLuint * params)
{
	(void) id;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetQueryivARB(GLenum target, GLenum pname, GLint * params)
{
	(void) target;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetRenderbufferParameterivEXT(GLenum target, GLenum pname, GLint * params)
{
	(void) target;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetShaderInfoLog(GLuint shader, GLsizei bufSize, GLsizei * length, char * infoLog)
{
	(void) shader;
	(void) bufSize;
	(void) length;
	(void) infoLog;
}

static void GLLOADER_APIENTRY NopGetShaderSource(GLuint shader, GLsizei bufSize, GLsizei * length, char * source)
{
	(void) shader;
	(void) bufSize;
	(void) length;
	(void) source;
}

static void GLLOADER_APIENTRY NopGetShaderiv(GLuint shader, GLenum pname, GLint * params)
{
	(void) shader;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetTrackMatrixivNV(GLenum target, GLuint address, GLenum pname, GLint * params)
{
	(void) target;
	(void) address;
	(void) pname;
	(void) params;
}

static GLint GLLOADER_APIENTRY NopGetUniformLocation(GLuint program, const char * name)
{
	(void) program;
	(void) name;
	return 0;
}

static void GLLOADER_APIENTRY NopGetUniformfv(GLuint program, GLint location, GLfloat * params)
{
	(void) program;
	(void) location;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetUniformiv(GLuint program, GLint location, GLint * params)
{
	(void) program;
	(void) location;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetUniformsLocations(GLuint program, GLsizei maxcbData, GLsizei * cbData, GLvoid * pData)
{
	(void) program;
	(void) maxcbData;
	(void) cbData;
	(void) pData;
}

static void GLLOADER_APIENTRY NopGetVertexAttribPointervARB(GLuint index, GLenum pname, GLvoid ** pointer)
{
	(void) index;
	(void) pname;
	(void) pointer;
}

static void GLLOADER_APIENTRY NopGetVertexAttribPointervNV(GLuint index, GLenum pname, GLvoid ** pointer)
{
	(void) index;
	(void) pname;
	(void) pointer;
}

static void GLLOADER_APIENTRY NopGetVertexAttribdvARB(GLuint index, GLenum pname, GLdouble * params)
{
	(void) index;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetVertexAttribdvNV(GLuint index, GLenum pname, GLdouble * params)
{
	(void) index;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetVertexAttribfvARB(GLuint index, GLenum pname, GLfloat * params)
{
	(void) index;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetVertexAttribfvNV(GLuint index, GLenum pname, GLfloat * params)
{
	(void) index;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetVertexAttribivARB(GLuint index, GLenum pname, GLint * params)
{
	(void) index;
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopGetVertexAttribivNV(GLuint index, GLenum pname, GLint * params)
{
	(void) index;
	(void) pname;
	(void) params;
}

static GLboolean GLLOADER_APIENTRY NopIsBufferARB(GLuint buffer)
{
	(void) buffer;
	return 0;
}

static GLboolean GLLOADER_APIENTRY NopIsFenceNV(GLuint fence)
{
	(void) fence;
	return 0;
}

static GLboolean GLLOADER_APIENTRY NopIsFramebufferEXT(GLuint framebuffer)
{
	(void) framebuffer;
	return 0;
}

static GLboolean GLLOADER_APIENTRY NopIsProgram(GLuint program)
{
	(void) program;
	return 0;
}

static GLboolean GLLOADER_APIENTRY NopIsProgramARB(GLuint program)
{
	(void) program;
	return 0;
}

static GLboolean GLLOADER_APIENTRY NopIsQueryARB(GLuint id)
{
	(void) id;
	return 0;
}

static GLboolean GLLOADER_APIENTRY NopIsRenderbufferEXT(GLuint renderbuffer)
{
	(void) renderbuffer;
	return 0;
}

static GLboolean GLLOADER_APIENTRY NopIsShader(GLuint shader)
{
	(void) shader;
	return 0;
}

static void GLLOADER_APIENTRY NopLinkProgram(GLuint program)
{
	(void) program;
}

static void GLLOADER_APIENTRY NopLoadProgramNV(GLenum target, GLuint id, GLsizei len, const GLubyte * program)
{
	(void) target;
	(void) id;
	(void) len;
	(void) program;
}

static void GLLOADER_APIENTRY NopLoadTransposeMatrixdARB(const GLdouble * m)
{
	(void) m;
}

static void GLLOADER_APIENTRY NopLoadTransposeMatrixfARB(const GLfloat * m)
{
	(void) m;
}

static void GLLOADER_APIENTRY NopLockArraysEXT(GLint first, GLint count)
{
	(void) first;
	(void) count;
}

static void GLLOADER_APIENTRY NopMakeCurrent(GLint window, GLint nativeWindow, GLint ctx)
{
	(void) window;
	(void) nativeWindow;
	(void) ctx;
}

static void * GLLOADER_APIENTRY NopMapBufferARB(GLenum target, GLenum access)
{
	(void) target;
	(void) access;
	return 0;
}

static void GLLOADER_APIENTRY NopMultTransposeMatrixdARB(const GLdouble * m)
{
	(void) m;
}

static void GLLOADER_APIENTRY NopMultTransposeMatrixfARB(const GLfloat * m)
{
	(void) m;
}

static void GLLOADER_APIENTRY NopMultiDrawArraysEXT(GLenum mode, GLint * first, GLsizei * count, GLsizei primcount)
{
	(void) mode;
	(void) first;
	(void) count;
	(void) primcount;
}

static void GLLOADER_APIENTRY NopMultiDrawElementsEXT(GLenum mode, const GLsizei * count, GLenum type, const GLvoid ** indices, GLsizei primcount)
{
	(void) mode;
	(void) count;
	(void) type;
	(void) indices;
	(void) primcount;
}

static void GLLOADER_APIENTRY NopMultiTexCoord1dARB(GLenum texture, GLdouble s)
{
	(void) texture;
	(void) s;
}

static void GLLOADER_APIENTRY NopMultiTexCoord1dvARB(GLenum texture, const GLdouble * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord1fARB(GLenum texture, GLfloat s)
{
	(void) texture;
	(void) s;
}

static void GLLOADER_APIENTRY NopMultiTexCoord1fvARB(GLenum texture, const GLfloat * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord1iARB(GLenum texture, GLint s)
{
	(void) texture;
	(void) s;
}

static void GLLOADER_APIENTRY NopMultiTexCoord1ivARB(GLenum texture, const GLint * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord1sARB(GLenum texture, GLshort s)
{
	(void) texture;
	(void) s;
}

static void GLLOADER_APIENTRY NopMultiTexCoord1svARB(GLenum texture, const GLshort * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord2dARB(GLenum texture, GLdouble s, GLdouble t)
{
	(void) texture;
	(void) s;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord2dvARB(GLenum texture, const GLdouble * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord2fARB(GLenum texture, GLfloat s, GLfloat t)
{
	(void) texture;
	(void) s;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord2fvARB(GLenum texture, const GLfloat * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord2iARB(GLenum texture, GLint s, GLint t)
{
	(void) texture;
	(void) s;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord2ivARB(GLenum texture, const GLint * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord2sARB(GLenum texture, GLshort s, GLshort t)
{
	(void) texture;
	(void) s;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord2svARB(GLenum texture, const GLshort * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord3dARB(GLenum texture, GLdouble s, GLdouble t, GLdouble r)
{
	(void) texture;
	(void) s;
	(void) t;
	(void) r;
}

static void GLLOADER_APIENTRY NopMultiTexCoord3dvARB(GLenum texture, const GLdouble * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord3fARB(GLenum texture, GLfloat s, GLfloat t, GLfloat r)
{
	(void) texture;
	(void) s;
	(void) t;
	(void) r;
}

static void GLLOADER_APIENTRY NopMultiTexCoord3fvARB(GLenum texture, const GLfloat * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord3iARB(GLenum texture, GLint s, GLint t, GLint r)
{
	(void) texture;
	(void) s;
	(void) t;
	(void) r;
}

static void GLLOADER_APIENTRY NopMultiTexCoord3ivARB(GLenum texture, const GLint * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord3sARB(GLenum texture, GLshort s, GLshort t, GLshort r)
{
	(void) texture;
	(void) s;
	(void) t;
	(void) r;
}

static void GLLOADER_APIENTRY NopMultiTexCoord3svARB(GLenum texture, const GLshort * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord4dARB(GLenum texture, GLdouble s, GLdouble t, GLdouble r, GLdouble q)
{
	(void) texture;
	(void) s;
	(void) t;
	(void) r;
	(void) q;
}

static void GLLOADER_APIENTRY NopMultiTexCoord4dvARB(GLenum texture, const GLdouble * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord4fARB(GLenum texture, GLfloat s, GLfloat t, GLfloat r, GLfloat q)
{
	(void) texture;
	(void) s;
	(void) t;
	(void) r;
	(void) q;
}

static void GLLOADER_APIENTRY NopMultiTexCoord4fvARB(GLenum texture, const GLfloat * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord4iARB(GLenum texture, GLint s, GLint t, GLint r, GLint q)
{
	(void) texture;
	(void) s;
	(void) t;
	(void) r;
	(void) q;
}

static void GLLOADER_APIENTRY NopMultiTexCoord4ivARB(GLenum texture, const GLint * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopMultiTexCoord4sARB(GLenum texture, GLshort s, GLshort t, GLshort r, GLshort q)
{
	(void) texture;
	(void) s;
	(void) t;
	(void) r;
	(void) q;
}

static void GLLOADER_APIENTRY NopMultiTexCoord4svARB(GLenum texture, const GLshort * t)
{
	(void) texture;
	(void) t;
}

static void GLLOADER_APIENTRY NopPointParameterfARB(GLenum pname, GLfloat param)
{
	(void) pname;
	(void) param;
}

static void GLLOADER_APIENTRY NopPointParameterfvARB(GLenum pname, const GLfloat * params)
{
	(void) pname;
	(void) params;
}

static void GLLOADER_APIENTRY NopPointParameteri(GLenum pname, GLint param)
{
	(void) pname;
	(void) param;
}

static void GLLOADER_APIENTRY NopPointParameteriv(GLenum pname, const GLint * param)
{
	(void) pname;
	(void) param;
}

static void GLLOADER_APIENTRY NopProgramEnvParameter4dARB(GLenum target, GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
{
	(void) target;
	(void) index;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopProgramEnvParameter4dvARB(GLenum target, GLuint index, const GLdouble * params)
{
	(void) target;
	(void) index;
	(void) params;
}

static void GLLOADER_APIENTRY NopProgramEnvParameter4fARB(GLenum target, GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
{
	(void) target;
	(void) index;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopProgramEnvParameter4fvARB(GLenum target, GLuint index, const GLfloat * params)
{
	(void) target;
	(void) index;
	(void) params;
}

static void GLLOADER_APIENTRY NopProgramLocalParameter4dARB(GLenum target, GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
{
	(void) target;
	(void) index;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopProgramLocalParameter4dvARB(GLenum target, GLuint index, const GLdouble * params)
{
	(void) target;
	(void) index;
	(void) params;
}

static void GLLOADER_APIENTRY NopProgramLocalParameter4fARB(GLenum target, GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
{
	(void) target;
	(void) index;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopProgramLocalParameter4fvARB(GLenum target, GLuint index, const GLfloat * params)
{
	(void) target;
	(void) index;
	(void) params;
}

static void GLLOADER_APIENTRY NopProgramNamedParameter4dNV(GLuint id, GLsizei len, const GLubyte * name, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
{
	(void) id;
	(void) len;
	(void) name;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopProgramNamedParameter4dvNV(GLuint id, GLsizei len, const GLubyte * name, const GLdouble * v)
{
	(void) id;
	(void) len;
	(void) name;
	(void) v;
}

static void GLLOADER_APIENTRY NopProgramNamedParameter4fNV(GLuint id, GLsizei len, const GLubyte * name, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
{
	(void) id;
	(void) len;
	(void) name;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopProgramNamedParameter4fvNV(GLuint id, GLsizei len, const GLubyte * name, const GLfloat * v)
{
	(void) id;
	(void) len;
	(void) name;
	(void) v;
}

static void GLLOADER_APIENTRY NopProgramParameter4dNV(GLenum target, GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
{
	(void) target;
	(void) index;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopProgramParameter4dvNV(GLenum target, GLuint index, const GLdouble * params)
{
	(void) target;
	(void) index;
	(void) params;
}

static void GLLOADER_APIENTRY NopProgramParameter4fNV(GLenum target, GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
{
	(void) target;
	(void) index;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopProgramParameter4fvNV(GLenum target, GLuint index, const GLfloat * params)
{
	(void) target;
	(void) index;
	(void) params;
}

static void GLLOADER_APIENTRY NopProgramParameters4dvNV(GLenum target, GLuint index, GLuint num, const GLdouble * params)
{
	(void) target;
	(void) index;
	(void) num;
	(void) params;
}

static void GLLOADER_APIENTRY NopProgramParameters4fvNV(GLenum target, GLuint index, GLuint num, const GLfloat * params)
{
	(void) target;
	(void) index;
	(void) num;
	(void) params;
}

static void GLLOADER_APIENTRY NopProgramStringARB(GLenum target, GLenum format, GLsizei len, const GLvoid * string)
{
	(void) target;
	(void) format;
	(void) len;
	(void) string;
}

static void GLLOADER_APIENTRY NopRenderbufferStorageEXT(GLenum target, GLenum internalformat, GLsizei width, GLsizei height)
{
	(void) target;
	(void) internalformat;
	(void) width;
	(void) height;
}

static void GLLOADER_APIENTRY NopRequestResidentProgramsNV(GLsizei n, const GLuint * ids)
{
	(void) n;
	(void) ids;
}

static void GLLOADER_APIENTRY NopSampleCoverageARB(GLclampf value, GLboolean invert)
{
	(void) value;
	(void) invert;
}

static void GLLOADER_APIENTRY NopSecondaryColor3bEXT(GLbyte red, GLbyte green, GLbyte blue)
{
	(void) red;
	(void) green;
	(void) blue;
}

static void GLLOADER_APIENTRY NopSecondaryColor3bvEXT(const GLbyte * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopSecondaryColor3dEXT(GLdouble red, GLdouble green, GLdouble blue)
{
	(void) red;
	(void) green;
	(void) blue;
}

static void GLLOADER_APIENTRY NopSecondaryColor3dvEXT(const GLdouble * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopSecondaryColor3fEXT(GLfloat red, GLfloat green, GLfloat blue)
{
	(void) red;
	(void) green;
	(void) blue;
}

static void GLLOADER_APIENTRY NopSecondaryColor3fvEXT(const GLfloat * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopSecondaryColor3iEXT(GLint red, GLint green, GLint blue)
{
	(void) red;
	(void) green;
	(void) blue;
}

static void GLLOADER_APIENTRY NopSecondaryColor3ivEXT(const GLint * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopSecondaryColor3sEXT(GLshort red, GLshort green, GLshort blue)
{
	(void) red;
	(void) green;
	(void) blue;
}

static void GLLOADER_APIENTRY NopSecondaryColor3svEXT(const GLshort * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopSecondaryColor3ubEXT(GLubyte red, GLubyte green, GLubyte blue)
{
	(void) red;
	(void) green;
	(void) blue;
}

static void GLLOADER_APIENTRY NopSecondaryColor3ubvEXT(const GLubyte * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopSecondaryColor3uiEXT(GLuint red, GLuint green, GLuint blue)
{
	(void) red;
	(void) green;
	(void) blue;
}

static void GLLOADER_APIENTRY NopSecondaryColor3uivEXT(const GLuint * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopSecondaryColor3usEXT(GLushort red, GLushort green, GLushort blue)
{
	(void) red;
	(void) green;
	(void) blue;
}

static void GLLOADER_APIENTRY NopSecondaryColor3usvEXT(const GLushort * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopSecondaryColorPointerEXT(GLint size, GLenum type, GLsizei stride, const GLvoid * pointer)
{
	(void) size;
	(void) type;
	(void) stride;
	(void) pointer;
}

static void GLLOADER_APIENTRY NopSemaphoreCreateCR(GLuint name, GLuint count)
{
	(void) name;
	(void) count;
}

static void GLLOADER_APIENTRY NopSemaphoreDestroyCR(GLuint name)
{
	(void) name;
}

static void GLLOADER_APIENTRY NopSemaphorePCR(GLuint name)
{
	(void) name;
}

static void GLLOADER_APIENTRY NopSemaphoreVCR(GLuint name)
{
	(void) name;
}

static void GLLOADER_APIENTRY NopSetFenceNV(GLuint fence, GLenum condition)
{
	(void) fence;
	(void) condition;
}

static void GLLOADER_APIENTRY NopShaderSource(GLuint shader, GLsizei count, const char ** string, const GLint * length)
{
	(void) shader;
	(void) count;
	(void) string;
	(void) length;
}

static void GLLOADER_APIENTRY NopStencilFuncSeparate(GLenum frontfunc, GLenum backfunc, GLint ref, GLuint mask)
{
	(void) frontfunc;
	(void) backfunc;
	(void) ref;
	(void) mask;
}

static void GLLOADER_APIENTRY NopStencilMaskSeparate(GLenum face, GLuint mask)
{
	(void) face;
	(void) mask;
}

static void GLLOADER_APIENTRY NopStencilOpSeparate(GLenum face, GLenum fail, GLenum zfail, GLenum zpass)
{
	(void) face;
	(void) fail;
	(void) zfail;
	(void) zpass;
}

static void GLLOADER_APIENTRY NopSwapBuffers(GLint window, GLint flags)
{
	(void) window;
	(void) flags;
}

static GLboolean GLLOADER_APIENTRY NopTestFenceNV(GLuint fence)
{
	(void) fence;
	return 0;
}

static void GLLOADER_APIENTRY NopTexImage3DEXT(GLenum target, GLint level, GLenum internalFormat, GLsizei width, GLsizei height, GLsizei depth, GLint border, GLenum format, GLenum type, const GLvoid * pixels)
{
	(void) target;
	(void) level;
	(void) internalFormat;
	(void) width;
	(void) height;
	(void) depth;
	(void) border;
	(void) format;
	(void) type;
	(void) pixels;
}

static void GLLOADER_APIENTRY NopTrackMatrixNV(GLenum target, GLuint address, GLenum matrix, GLenum transform)
{
	(void) target;
	(void) address;
	(void) matrix;
	(void) transform;
}

static void GLLOADER_APIENTRY NopUniform1f(GLint location, GLfloat v0)
{
	(void) location;
	(void) v0;
}

static void GLLOADER_APIENTRY NopUniform1fv(GLint location, GLsizei count, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniform1i(GLint location, GLint v0)
{
	(void) location;
	(void) v0;
}

static void GLLOADER_APIENTRY NopUniform1iv(GLint location, GLsizei count, const GLint * value)
{
	(void) location;
	(void) count;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniform2f(GLint location, GLfloat v0, GLfloat v1)
{
	(void) location;
	(void) v0;
	(void) v1;
}

static void GLLOADER_APIENTRY NopUniform2fv(GLint location, GLsizei count, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniform2i(GLint location, GLint v0, GLint v1)
{
	(void) location;
	(void) v0;
	(void) v1;
}

static void GLLOADER_APIENTRY NopUniform2iv(GLint location, GLsizei count, const GLint * value)
{
	(void) location;
	(void) count;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniform3f(GLint location, GLfloat v0, GLfloat v1, GLfloat v2)
{
	(void) location;
	(void) v0;
	(void) v1;
	(void) v2;
}

static void GLLOADER_APIENTRY NopUniform3fv(GLint location, GLsizei count, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniform3i(GLint location, GLint v0, GLint v1, GLint v2)
{
	(void) location;
	(void) v0;
	(void) v1;
	(void) v2;
}

static void GLLOADER_APIENTRY NopUniform3iv(GLint location, GLsizei count, const GLint * value)
{
	(void) location;
	(void) count;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniform4f(GLint location, GLfloat v0, GLfloat v1, GLfloat v2, GLfloat v3)
{
	(void) location;
	(void) v0;
	(void) v1;
	(void) v2;
	(void) v3;
}

static void GLLOADER_APIENTRY NopUniform4fv(GLint location, GLsizei count, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniform4i(GLint location, GLint v0, GLint v1, GLint v2, GLint v3)
{
	(void) location;
	(void) v0;
	(void) v1;
	(void) v2;
	(void) v3;
}

static void GLLOADER_APIENTRY NopUniform4iv(GLint location, GLsizei count, const GLint * value)
{
	(void) location;
	(void) count;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniformMatrix2fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) transpose;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniformMatrix2x3fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) transpose;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniformMatrix2x4fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) transpose;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniformMatrix3fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) transpose;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniformMatrix3x2fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) transpose;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniformMatrix3x4fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) transpose;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniformMatrix4fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) transpose;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniformMatrix4x2fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) transpose;
	(void) value;
}

static void GLLOADER_APIENTRY NopUniformMatrix4x3fv(GLint location, GLsizei count, GLboolean transpose, const GLfloat * value)
{
	(void) location;
	(void) count;
	(void) transpose;
	(void) value;
}

static void GLLOADER_APIENTRY NopUnlockArraysEXT(void)
{
}

static GLboolean GLLOADER_APIENTRY NopUnmapBufferARB(GLenum target)
{
	(void) target;
	return 0;
}

static void GLLOADER_APIENTRY NopUseProgram(GLuint program)
{
	(void) program;
}

static void GLLOADER_APIENTRY NopVBoxAttachThread(void)
{
}

static GLint GLLOADER_APIENTRY NopVBoxConCreate(struct VBOXUHGSMI * pHgsmi)
{
	(void) pHgsmi;
	return 0;
}

static void GLLOADER_APIENTRY NopVBoxConDestroy(GLint con)
{
	(void) con;
}

static void GLLOADER_APIENTRY NopVBoxConFlush(GLint con)
{
	(void) con;
}

static GLint GLLOADER_APIENTRY NopVBoxCreateContext(GLint con, const char * dpyName, GLint visual, GLint shareCtx)
{
	(void) con;
	(void) dpyName;
	(void) visual;
	(void) shareCtx;
	return 0;
}

static void GLLOADER_APIENTRY NopVBoxDetachThread(void)
{
}

static GLuint GLLOADER_APIENTRY NopVBoxPackGetInjectID(GLint con)
{
	(void) con;
	return 0;
}

static void GLLOADER_APIENTRY NopVBoxPackSetInjectID(GLuint id)
{
	(void) id;
}

static GLint GLLOADER_APIENTRY NopVBoxPackSetInjectThread(struct VBOXUHGSMI * pHgsmi)
{
	(void) pHgsmi;
	return 0;
}

static GLint GLLOADER_APIENTRY NopVBoxWindowCreate(GLint con, const char * dpyName, GLint visBits)
{
	(void) con;
	(void) dpyName;
	(void) visBits;
	return 0;
}

static void GLLOADER_APIENTRY NopVBoxWindowDestroy(GLint con, GLint window)
{
	(void) con;
	(void) window;
}

static void GLLOADER_APIENTRY NopValidateProgram(GLuint program)
{
	(void) program;
}

static void GLLOADER_APIENTRY NopVertexArrayRangeNV(GLsizei length, const GLvoid * pointer)
{
	(void) length;
	(void) pointer;
}

static void GLLOADER_APIENTRY NopVertexAttrib1dARB(GLuint index, GLdouble x)
{
	(void) index;
	(void) x;
}

static void GLLOADER_APIENTRY NopVertexAttrib1dvARB(GLuint index, const GLdouble * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib1fARB(GLuint index, GLfloat x)
{
	(void) index;
	(void) x;
}

static void GLLOADER_APIENTRY NopVertexAttrib1fvARB(GLuint index, const GLfloat * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib1sARB(GLuint index, GLshort x)
{
	(void) index;
	(void) x;
}

static void GLLOADER_APIENTRY NopVertexAttrib1svARB(GLuint index, const GLshort * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib2dARB(GLuint index, GLdouble x, GLdouble y)
{
	(void) index;
	(void) x;
	(void) y;
}

static void GLLOADER_APIENTRY NopVertexAttrib2dvARB(GLuint index, const GLdouble * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib2fARB(GLuint index, GLfloat x, GLfloat y)
{
	(void) index;
	(void) x;
	(void) y;
}

static void GLLOADER_APIENTRY NopVertexAttrib2fvARB(GLuint index, const GLfloat * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib2sARB(GLuint index, GLshort x, GLshort y)
{
	(void) index;
	(void) x;
	(void) y;
}

static void GLLOADER_APIENTRY NopVertexAttrib2svARB(GLuint index, const GLshort * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib3dARB(GLuint index, GLdouble x, GLdouble y, GLdouble z)
{
	(void) index;
	(void) x;
	(void) y;
	(void) z;
}

static void GLLOADER_APIENTRY NopVertexAttrib3dvARB(GLuint index, const GLdouble * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib3fARB(GLuint index, GLfloat x, GLfloat y, GLfloat z)
{
	(void) index;
	(void) x;
	(void) y;
	(void) z;
}

static void GLLOADER_APIENTRY NopVertexAttrib3fvARB(GLuint index, const GLfloat * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib3sARB(GLuint index, GLshort x, GLshort y, GLshort z)
{
	(void) index;
	(void) x;
	(void) y;
	(void) z;
}

static void GLLOADER_APIENTRY NopVertexAttrib3svARB(GLuint index, const GLshort * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4NbvARB(GLuint index, const GLbyte * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4NivARB(GLuint index, const GLint * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4NsvARB(GLuint index, const GLshort * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4NubARB(GLuint index, GLubyte x, GLubyte y, GLubyte z, GLubyte w)
{
	(void) index;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopVertexAttrib4NubvARB(GLuint index, const GLubyte * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4NuivARB(GLuint index, const GLuint * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4NusvARB(GLuint index, const GLushort * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4bvARB(GLuint index, const GLbyte * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4dARB(GLuint index, GLdouble x, GLdouble y, GLdouble z, GLdouble w)
{
	(void) index;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopVertexAttrib4dvARB(GLuint index, const GLdouble * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4fARB(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
{
	(void) index;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopVertexAttrib4fvARB(GLuint index, const GLfloat * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4ivARB(GLuint index, const GLint * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4sARB(GLuint index, GLshort x, GLshort y, GLshort z, GLshort w)
{
	(void) index;
	(void) x;
	(void) y;
	(void) z;
	(void) w;
}

static void GLLOADER_APIENTRY NopVertexAttrib4svARB(GLuint index, const GLshort * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4ubvARB(GLuint index, const GLubyte * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4uivARB(GLuint index, const GLuint * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttrib4usvARB(GLuint index, const GLushort * v)
{
	(void) index;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribPointerARB(GLuint index, GLint size, GLenum type, GLboolean normalized, GLsizei stride, const GLvoid * pointer)
{
	(void) index;
	(void) size;
	(void) type;
	(void) normalized;
	(void) stride;
	(void) pointer;
}

static void GLLOADER_APIENTRY NopVertexAttribPointerNV(GLuint index, GLint size, GLenum type, GLsizei stride, const GLvoid * pointer)
{
	(void) index;
	(void) size;
	(void) type;
	(void) stride;
	(void) pointer;
}

static void GLLOADER_APIENTRY NopVertexAttribs1dvNV(GLuint index, GLsizei n, const GLdouble * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs1fvNV(GLuint index, GLsizei n, const GLfloat * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs1svNV(GLuint index, GLsizei n, const GLshort * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs2dvNV(GLuint index, GLsizei n, const GLdouble * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs2fvNV(GLuint index, GLsizei n, const GLfloat * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs2svNV(GLuint index, GLsizei n, const GLshort * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs3dvNV(GLuint index, GLsizei n, const GLdouble * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs3fvNV(GLuint index, GLsizei n, const GLfloat * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs3svNV(GLuint index, GLsizei n, const GLshort * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs4dvNV(GLuint index, GLsizei n, const GLdouble * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs4fvNV(GLuint index, GLsizei n, const GLfloat * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs4svNV(GLuint index, GLsizei n, const GLshort * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static void GLLOADER_APIENTRY NopVertexAttribs4ubvNV(GLuint index, GLsizei n, const GLubyte * v)
{
	(void) index;
	(void) n;
	(void) v;
}

static GLint GLLOADER_APIENTRY NopWindowCreate(const char * dpyName, GLint visBits)
{
	(void) dpyName;
	(void) visBits;
	return 0;
}

static void GLLOADER_APIENTRY NopWindowDestroy(GLint window)
{
	(void) window;
}

static void GLLOADER_APIENTRY NopWindowPos2dARB(GLdouble x, GLdouble y)
{
	(void) x;
	(void) y;
}

static void GLLOADER_APIENTRY NopWindowPos2dvARB(const GLdouble * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopWindowPos2fARB(GLfloat x, GLfloat y)
{
	(void) x;
	(void) y;
}

static void GLLOADER_APIENTRY NopWindowPos2fvARB(const GLfloat * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopWindowPos2iARB(GLint x, GLint y)
{
	(void) x;
	(void) y;
}

static void GLLOADER_APIENTRY NopWindowPos2ivARB(const GLint * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopWindowPos2sARB(GLshort x, GLshort y)
{
	(void) x;
	(void) y;
}

static void GLLOADER_APIENTRY NopWindowPos2svARB(const GLshort * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopWindowPos3dARB(GLdouble x, GLdouble y, GLdouble z)
{
	(void) x;
	(void) y;
	(void) z;
}

static void GLLOADER_APIENTRY NopWindowPos3dvARB(const GLdouble * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopWindowPos3fARB(GLfloat x, GLfloat y, GLfloat z)
{
	(void) x;
	(void) y;
	(void) z;
}

static void GLLOADER_APIENTRY NopWindowPos3fvARB(const GLfloat * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopWindowPos3iARB(GLint x, GLint y, GLint z)
{
	(void) x;
	(void) y;
	(void) z;
}

static void GLLOADER_APIENTRY NopWindowPos3ivARB(const GLint * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopWindowPos3sARB(GLshort x, GLshort y, GLshort z)
{
	(void) x;
	(void) y;
	(void) z;
}

static void GLLOADER_APIENTRY NopWindowPos3svARB(const GLshort * v)
{
	(void) v;
}

static void GLLOADER_APIENTRY NopWindowPosition(GLint window, GLint x, GLint y)
{
	(void) window;
	(void) x;
	(void) y;
}

static void GLLOADER_APIENTRY NopWindowShow(GLint window, GLint flag)
{
	(void) window;
	(void) flag;
}

static void GLLOADER_APIENTRY NopWindowSize(GLint window, GLint w, GLint h)
{
	(void) window;
	(void) w;
	(void) h;
}

static void GLLOADER_APIENTRY NopWindowVisibleRegion(GLint window, GLint cRects, GLint * pRects)
{
	(void) window;
	(void) cRects;
	(void) pRects;
}

static void GLLOADER_APIENTRY NopWriteback(GLint * writeback)
{
	(void) writeback;
}

static void GLLOADER_APIENTRY NopZPixCR(GLsizei width, GLsizei height, GLenum format, GLenum type, GLenum ztype, GLint zparm, GLint length, const GLvoid * pixels)
{
	(void) width;
	(void) height;
	(void) format;
	(void) type;
	(void) ztype;
	(void) zparm;
	(void) length;
	(void) pixels;
}


void
crUnloadOpenGL( void )
{
	crDLLClose( glDll );
	glDll = NULL;

#ifdef DARWIN
	crDLLClose( cglDll );
	cglDll = NULL;

# ifndef VBOX_WITH_COCOA_QT
	crDLLClose( aglDll );
	aglDll = NULL;
# endif
#endif
}

/*
 * Initialize the 'interface' structure with the WGL or GLX window system
 * interface functions.
 * Then, fill in the table with (name, pointer) pairs for all the core
 * OpenGL entrypoint functions.  But only if table is not NULL
 * Return: number of entries placed in table[], or 0 if error.
 */
int
crLoadOpenGL( crOpenGLInterface *interface, SPUNamedFunctionTable table[] )
{
	static const char *coreFunctions[] = {

		"glAccum",
		"glAlphaFunc",
		"glAreTexturesResident",
		"glArrayElement",
		"glBegin",
		"glBindTexture",
		"glBitmap",
		"glBlendFunc",
		"glCallList",
		"glCallLists",
		"glClear",
		"glClearAccum",
		"glClearColor",
		"glClearDepth",
		"glClearIndex",
		"glClearStencil",
		"glClipPlane",
		"glColor3b",
		"glColor3bv",
		"glColor3d",
		"glColor3dv",
		"glColor3f",
		"glColor3fv",
		"glColor3i",
		"glColor3iv",
		"glColor3s",
		"glColor3sv",
		"glColor3ub",
		"glColor3ubv",
		"glColor3ui",
		"glColor3uiv",
		"glColor3us",
		"glColor3usv",
		"glColor4b",
		"glColor4bv",
		"glColor4d",
		"glColor4dv",
		"glColor4f",
		"glColor4fv",
		"glColor4i",
		"glColor4iv",
		"glColor4s",
		"glColor4sv",
		"glColor4ub",
		"glColor4ubv",
		"glColor4ui",
		"glColor4uiv",
		"glColor4us",
		"glColor4usv",
		"glColorMask",
		"glColorMaterial",
		"glColorPointer",
		"glCopyPixels",
		"glCopyTexImage1D",
		"glCopyTexImage2D",
		"glCopyTexSubImage1D",
		"glCopyTexSubImage2D",
		"glCopyTexSubImage3D",
		"glCullFace",
		"glDeleteLists",
		"glDeleteTextures",
		"glDepthFunc",
		"glDepthMask",
		"glDepthRange",
		"glDisable",
		"glDisableClientState",
		"glDrawArrays",
		"glDrawBuffer",
		"glDrawElements",
		"glDrawPixels",
		"glDrawRangeElements",
		"glEdgeFlag",
		"glEdgeFlagPointer",
		"glEdgeFlagv",
		"glEnable",
		"glEnableClientState",
		"glEnd",
		"glEndList",
		"glEvalCoord1d",
		"glEvalCoord1dv",
		"glEvalCoord1f",
		"glEvalCoord1fv",
		"glEvalCoord2d",
		"glEvalCoord2dv",
		"glEvalCoord2f",
		"glEvalCoord2fv",
		"glEvalMesh1",
		"glEvalMesh2",
		"glEvalPoint1",
		"glEvalPoint2",
		"glFeedbackBuffer",
		"glFinish",
		"glFlush",
		"glFogf",
		"glFogfv",
		"glFogi",
		"glFogiv",
		"glFrontFace",
		"glFrustum",
		"glGenLists",
		"glGenTextures",
		"glGetBooleanv",
		"glGetClipPlane",
		"glGetDoublev",
		"glGetError",
		"glGetFloatv",
		"glGetIntegerv",
		"glGetLightfv",
		"glGetLightiv",
		"glGetMapdv",
		"glGetMapfv",
		"glGetMapiv",
		"glGetMaterialfv",
		"glGetMaterialiv",
		"glGetPixelMapfv",
		"glGetPixelMapuiv",
		"glGetPixelMapusv",
		"glGetPointerv",
		"glGetPolygonStipple",
		"glGetString",
		"glGetTexEnvfv",
		"glGetTexEnviv",
		"glGetTexGendv",
		"glGetTexGenfv",
		"glGetTexGeniv",
		"glGetTexImage",
		"glGetTexLevelParameterfv",
		"glGetTexLevelParameteriv",
		"glGetTexParameterfv",
		"glGetTexParameteriv",
		"glHint",
		"glIndexMask",
		"glIndexPointer",
		"glIndexd",
		"glIndexdv",
		"glIndexf",
		"glIndexfv",
		"glIndexi",
		"glIndexiv",
		"glIndexs",
		"glIndexsv",
		"glIndexub",
		"glIndexubv",
		"glInitNames",
		"glInterleavedArrays",
		"glIsEnabled",
		"glIsList",
		"glIsTexture",
		"glLightModelf",
		"glLightModelfv",
		"glLightModeli",
		"glLightModeliv",
		"glLightf",
		"glLightfv",
		"glLighti",
		"glLightiv",
		"glLineStipple",
		"glLineWidth",
		"glListBase",
		"glLoadIdentity",
		"glLoadMatrixd",
		"glLoadMatrixf",
		"glLoadName",
		"glLogicOp",
		"glMap1d",
		"glMap1f",
		"glMap2d",
		"glMap2f",
		"glMapGrid1d",
		"glMapGrid1f",
		"glMapGrid2d",
		"glMapGrid2f",
		"glMaterialf",
		"glMaterialfv",
		"glMateriali",
		"glMaterialiv",
		"glMatrixMode",
		"glMultMatrixd",
		"glMultMatrixf",
		"glNewList",
		"glNormal3b",
		"glNormal3bv",
		"glNormal3d",
		"glNormal3dv",
		"glNormal3f",
		"glNormal3fv",
		"glNormal3i",
		"glNormal3iv",
		"glNormal3s",
		"glNormal3sv",
		"glNormalPointer",
		"glOrtho",
		"glPassThrough",
		"glPixelMapfv",
		"glPixelMapuiv",
		"glPixelMapusv",
		"glPixelStoref",
		"glPixelStorei",
		"glPixelTransferf",
		"glPixelTransferi",
		"glPixelZoom",
		"glPointSize",
		"glPolygonMode",
		"glPolygonOffset",
		"glPolygonStipple",
		"glPopAttrib",
		"glPopClientAttrib",
		"glPopMatrix",
		"glPopName",
		"glPrioritizeTextures",
		"glPushAttrib",
		"glPushClientAttrib",
		"glPushMatrix",
		"glPushName",
		"glRasterPos2d",
		"glRasterPos2dv",
		"glRasterPos2f",
		"glRasterPos2fv",
		"glRasterPos2i",
		"glRasterPos2iv",
		"glRasterPos2s",
		"glRasterPos2sv",
		"glRasterPos3d",
		"glRasterPos3dv",
		"glRasterPos3f",
		"glRasterPos3fv",
		"glRasterPos3i",
		"glRasterPos3iv",
		"glRasterPos3s",
		"glRasterPos3sv",
		"glRasterPos4d",
		"glRasterPos4dv",
		"glRasterPos4f",
		"glRasterPos4fv",
		"glRasterPos4i",
		"glRasterPos4iv",
		"glRasterPos4s",
		"glRasterPos4sv",
		"glReadBuffer",
		"glReadPixels",
		"glRectd",
		"glRectdv",
		"glRectf",
		"glRectfv",
		"glRecti",
		"glRectiv",
		"glRects",
		"glRectsv",
		"glRenderMode",
		"glRotated",
		"glRotatef",
		"glScaled",
		"glScalef",
		"glScissor",
		"glSelectBuffer",
		"glShadeModel",
		"glStencilFunc",
		"glStencilMask",
		"glStencilOp",
		"glTexCoord1d",
		"glTexCoord1dv",
		"glTexCoord1f",
		"glTexCoord1fv",
		"glTexCoord1i",
		"glTexCoord1iv",
		"glTexCoord1s",
		"glTexCoord1sv",
		"glTexCoord2d",
		"glTexCoord2dv",
		"glTexCoord2f",
		"glTexCoord2fv",
		"glTexCoord2i",
		"glTexCoord2iv",
		"glTexCoord2s",
		"glTexCoord2sv",
		"glTexCoord3d",
		"glTexCoord3dv",
		"glTexCoord3f",
		"glTexCoord3fv",
		"glTexCoord3i",
		"glTexCoord3iv",
		"glTexCoord3s",
		"glTexCoord3sv",
		"glTexCoord4d",
		"glTexCoord4dv",
		"glTexCoord4f",
		"glTexCoord4fv",
		"glTexCoord4i",
		"glTexCoord4iv",
		"glTexCoord4s",
		"glTexCoord4sv",
		"glTexCoordPointer",
		"glTexEnvf",
		"glTexEnvfv",
		"glTexEnvi",
		"glTexEnviv",
		"glTexGend",
		"glTexGendv",
		"glTexGenf",
		"glTexGenfv",
		"glTexGeni",
		"glTexGeniv",
		"glTexImage1D",
		"glTexImage2D",
		"glTexImage3D",
		"glTexParameterf",
		"glTexParameterfv",
		"glTexParameteri",
		"glTexParameteriv",
		"glTexSubImage1D",
		"glTexSubImage2D",
		"glTexSubImage3D",
		"glTranslated",
		"glTranslatef",
		"glVertex2d",
		"glVertex2dv",
		"glVertex2f",
		"glVertex2fv",
		"glVertex2i",
		"glVertex2iv",
		"glVertex2s",
		"glVertex2sv",
		"glVertex3d",
		"glVertex3dv",
		"glVertex3f",
		"glVertex3fv",
		"glVertex3i",
		"glVertex3iv",
		"glVertex3s",
		"glVertex3sv",
		"glVertex4d",
		"glVertex4dv",
		"glVertex4f",
		"glVertex4fv",
		"glVertex4i",
		"glVertex4iv",
		"glVertex4s",
		"glVertex4sv",
		"glVertexPointer",
		"glViewport",

		NULL
	};
	SPUNamedFunctionTable *entry = table;
	int i;

	const char *env_syspath = crGetenv( "CR_SYSTEM_GL_PATH" );
#ifdef DARWIN
	const char *env_cgl_syspath = crGetenv( "CR_SYSTEM_CGL_PATH" );
# ifndef VBOX_WITH_COCOA_QT
	const char *env_agl_syspath = crGetenv( "CR_SYSTEM_AGL_PATH" );
# endif
#endif

	crDebug( "Looking for the system's OpenGL library..." );
#ifdef DARWIN
	glDll = __findSystemGL( env_syspath, SYSTEM_GL_LIB_DIR, SYSTEM_GL );
#else
	glDll = __findSystemGL( env_syspath );
#endif
	if (!glDll)
	{
		crError("Unable to find system OpenGL!");
		return 0;
	}

	crDebug( "Found it in %s.", !env_syspath ? "default path" : env_syspath );

#ifdef DARWIN
	crDebug( "Looking for the system's CGL library..." );
	cglDll = __findSystemGL( env_cgl_syspath, SYSTEM_CGL_DIR, SYSTEM_CGL );
	if (!cglDll)
	{
		crError("Unable to find system CGL!");
		return 0;
	}

	crDebug( "Found it in %s.", !env_cgl_syspath ? "default path" : env_cgl_syspath );

# ifndef VBOX_WITH_COCOA_QT
	crDebug( "Looking for the system's AGL library..." );
	aglDll = __findSystemGL( env_agl_syspath, SYSTEM_AGL_DIR, SYSTEM_AGL );
	if (!aglDll)
	{
		crError("Unable to find system AGL!");
		return 0;
	}

	crDebug( "Found it in %s.", !env_agl_syspath ? "default path" : env_agl_syspath );
# endif
#endif

#ifdef WINDOWS
	interface->wglGetProcAddress = (wglGetProcAddressFunc_t) crDLLGetNoError( glDll, "wglGetProcAddress" );
	interface->wglMakeCurrent = (wglMakeCurrentFunc_t) crDLLGetNoError( glDll, "wglMakeCurrent" );
	interface->wglSwapBuffers = (wglSwapBuffersFunc_t) crDLLGetNoError( glDll, "wglSwapBuffers" );
	interface->wglCreateContext = (wglCreateContextFunc_t) crDLLGetNoError( glDll, "wglCreateContext" );
	interface->wglDeleteContext = (wglDeleteContextFunc_t) crDLLGetNoError( glDll, "wglDeleteContext" );
	interface->wglShareLists = (wglShareListsFunc_t) crDLLGetNoError( glDll, "wglShareLists" );
	interface->wglGetCurrentContext = (wglGetCurrentContextFunc_t) crDLLGetNoError( glDll, "wglGetCurrentContext" );
	interface->wglChoosePixelFormat = (wglChoosePixelFormatFunc_t) crDLLGetNoError( glDll, "wglChoosePixelFormat" );
	interface->wglDescribePixelFormat = (wglDescribePixelFormatFunc_t) crDLLGetNoError( glDll, "wglDescribePixelFormat" );
	interface->wglSetPixelFormat = (wglSetPixelFormatFunc_t) crDLLGetNoError( glDll, "wglSetPixelFormat" );
	interface->wglChoosePixelFormatEXT = (wglChoosePixelFormatEXTFunc_t) crDLLGetNoError( glDll, "wglChoosePixelFormatEXT" );
	interface->wglGetPixelFormatAttribivEXT = (wglGetPixelFormatAttribivEXTFunc_t) crDLLGetNoError( glDll, "wglGetPixelFormatAttribivEXT" );
	interface->wglGetPixelFormatAttribfvEXT = (wglGetPixelFormatAttribfvEXTFunc_t) crDLLGetNoError( glDll, "wglGetPixelFormatAttribfvEXT" );
	interface->glGetString = (glGetStringFunc_t) crDLLGetNoError( glDll, "glGetString" );
#elif defined(DARWIN)
# ifndef VBOX_WITH_COCOA_QT
	interface->aglCreateContext = (aglCreateContextFunc_t) crDLLGetNoError( aglDll, "aglCreateContext" );
	interface->aglDestroyContext = (aglDestroyContextFunc_t) crDLLGetNoError( aglDll, "aglDestroyContext" );
	interface->aglSetCurrentContext = (aglSetCurrentContextFunc_t) crDLLGetNoError( aglDll, "aglSetCurrentContext" );
	interface->aglSwapBuffers = (aglSwapBuffersFunc_t) crDLLGetNoError( aglDll, "aglSwapBuffers" );
	interface->aglChoosePixelFormat = (aglChoosePixelFormatFunc_t) crDLLGetNoError( aglDll, "aglChoosePixelFormat" );
	interface->aglDestroyPixelFormat = (aglDestroyPixelFormatFunc_t) crDLLGetNoError( aglDll, "aglDestroyPixelFormat" );
	interface->aglDescribePixelFormat = (aglDescribePixelFormatFunc_t) crDLLGetNoError( aglDll, "aglDescribePixelFormat" );
	interface->aglGetCurrentContext = (aglGetCurrentContextFunc_t) crDLLGetNoError( aglDll, "aglGetCurrentContext" );
	interface->aglSetDrawable = (aglSetDrawableFunc_t) crDLLGetNoError( aglDll, "aglSetDrawable" );
	interface->aglGetDrawable = (aglGetDrawableFunc_t) crDLLGetNoError( aglDll, "aglGetDrawable" );
	interface->aglSetFullScreen = (aglSetFullScreenFunc_t) crDLLGetNoError( aglDll, "aglSetFullScreen" );
	interface->aglUpdateContext = (aglUpdateContextFunc_t) crDLLGetNoError( aglDll, "aglUpdateContext" );
	interface->aglUseFont = (aglUseFontFunc_t) crDLLGetNoError( aglDll, "aglUseFont" );
	interface->aglSetInteger = (aglSetIntegerFunc_t) crDLLGetNoError( aglDll, "aglSetInteger" );
	interface->aglGetInteger = (aglGetIntegerFunc_t) crDLLGetNoError( aglDll, "aglGetInteger" );
	interface->aglGetError = (aglGetErrorFunc_t) crDLLGetNoError( aglDll, "aglGetError" );
	interface->aglEnable = (aglEnableFunc_t) crDLLGetNoError( aglDll, "aglEnable" );
	interface->aglDisable = (aglDisableFunc_t) crDLLGetNoError( aglDll, "aglDisable" );
# endif
	interface->CGLChoosePixelFormat = (CGLChoosePixelFormatFunc_t) crDLLGetNoError( cglDll, "CGLChoosePixelFormat" );
	interface->CGLDestroyPixelFormat = (CGLDestroyPixelFormatFunc_t) crDLLGetNoError( cglDll, "CGLDestroyPixelFormat" );
	interface->CGLDescribePixelFormat = (CGLDescribePixelFormatFunc_t) crDLLGetNoError( cglDll, "CGLDescribePixelFormat" );
	interface->CGLQueryRendererInfo = (CGLQueryRendererInfoFunc_t) crDLLGetNoError( cglDll, "CGLQueryRendererInfo" );
	interface->CGLDestroyRendererInfo = (CGLDestroyRendererInfoFunc_t) crDLLGetNoError( cglDll, "CGLDestroyRendererInfo" );
	interface->CGLDescribeRenderer = (CGLDescribeRendererFunc_t) crDLLGetNoError( cglDll, "CGLDescribeRenderer" );
	interface->CGLCreateContext = (CGLCreateContextFunc_t) crDLLGetNoError( cglDll, "CGLCreateContext" );
	interface->CGLDestroyContext = (CGLDestroyContextFunc_t) crDLLGetNoError( cglDll, "CGLDestroyContext" );
	interface->CGLCopyContext = (CGLCopyContextFunc_t) crDLLGetNoError( cglDll, "CGLCopyContext" );
	interface->CGLCreatePBuffer = (CGLCreatePBufferFunc_t) crDLLGetNoError( cglDll, "CGLCreatePBuffer" );
	interface->CGLDestroyPBuffer = (CGLDestroyPBufferFunc_t) crDLLGetNoError( cglDll, "CGLDestroyPBuffer" );
	interface->CGLDescribePBuffer = (CGLDescribePBufferFunc_t) crDLLGetNoError( cglDll, "CGLDescribePBuffer" );
	interface->CGLTexImagePBuffer = (CGLTexImagePBufferFunc_t) crDLLGetNoError( cglDll, "CGLTexImagePBuffer" );
	interface->CGLSetOffScreen = (CGLSetOffScreenFunc_t) crDLLGetNoError( cglDll, "CGLSetOffScreen" );
	interface->CGLGetOffScreen = (CGLGetOffScreenFunc_t) crDLLGetNoError( cglDll, "CGLGetOffScreen" );
	interface->CGLSetFullScreen = (CGLSetFullScreenFunc_t) crDLLGetNoError( cglDll, "CGLSetFullScreen" );
	interface->CGLSetPBuffer = (CGLSetPBufferFunc_t) crDLLGetNoError( cglDll, "CGLSetPBuffer" );
	interface->CGLGetPBuffer = (CGLGetPBufferFunc_t) crDLLGetNoError( cglDll, "CGLGetPBuffer" );
	interface->CGLClearDrawable = (CGLClearDrawableFunc_t) crDLLGetNoError( cglDll, "CGLClearDrawable" );
	interface->CGLFlushDrawable = (CGLFlushDrawableFunc_t) crDLLGetNoError( cglDll, "CGLFlushDrawable" );
	interface->CGLEnable = (CGLEnableFunc_t) crDLLGetNoError( cglDll, "CGLEnable" );
	interface->CGLDisable = (CGLDisableFunc_t) crDLLGetNoError( cglDll, "CGLDisable" );
	interface->CGLIsEnabled = (CGLIsEnabledFunc_t) crDLLGetNoError( cglDll, "CGLIsEnabled" );
	interface->CGLSetParameter = (CGLSetParameterFunc_t) crDLLGetNoError( cglDll, "CGLSetParameter" );
	interface->CGLGetParameter = (CGLGetParameterFunc_t) crDLLGetNoError( cglDll, "CGLGetParameter" );
	interface->CGLSetVirtualScreen = (CGLSetVirtualScreenFunc_t) crDLLGetNoError( cglDll, "CGLSetVirtualScreen" );
	interface->CGLGetVirtualScreen = (CGLGetVirtualScreenFunc_t) crDLLGetNoError( cglDll, "CGLGetVirtualScreen" );
	interface->CGLSetOption = (CGLSetOptionFunc_t) crDLLGetNoError( cglDll, "CGLSetOption" );
	interface->CGLGetOption = (CGLGetOptionFunc_t) crDLLGetNoError( cglDll, "CGLGetOption" );
	interface->CGLGetVersion = (CGLGetVersionFunc_t) crDLLGetNoError( cglDll, "CGLGetVersion" );
	interface->CGLErrorString = (CGLErrorStringFunc_t) crDLLGetNoError( cglDll, "CGLErrorString" );
	interface->CGLSetSurface = (CGLSetSurfaceFunc_t) crDLLGetNoError( cglDll, "CGLSetSurface" );
	interface->CGLGetSurface = (CGLGetSurfaceFunc_t) crDLLGetNoError( cglDll, "CGLGetSurface" );
	interface->CGLUpdateContext = (CGLUpdateContextFunc_t) crDLLGetNoError( cglDll, "CGLUpdateContext" );
	interface->glGetString = (glGetStringFunc_t) crDLLGetNoError( cglDll, "glGetString" );
	interface->CGLGetCurrentContext = (CGLGetCurrentContextFunc_t) crDLLGetNoError( glDll, "CGLGetCurrentContext" );
	interface->CGLSetCurrentContext = (CGLSetCurrentContextFunc_t) crDLLGetNoError( glDll, "CGLSetCurrentContext" );
#else
	/* GLX */
	interface->glXGetConfig = (glXGetConfigFunc_t) crDLLGetNoError( glDll, "glXGetConfig" );
	interface->glXQueryExtension = (glXQueryExtensionFunc_t) crDLLGetNoError( glDll, "glXQueryExtension" );
	interface->glXQueryVersion = (glXQueryVersionFunc_t) crDLLGetNoError( glDll, "glXQueryVersion" );
	interface->glXQueryExtensionsString = (glXQueryExtensionsStringFunc_t) crDLLGetNoError( glDll, "glXQueryExtensionsString" );
	interface->glXChooseVisual = (glXChooseVisualFunc_t) crDLLGetNoError( glDll, "glXChooseVisual" );
	interface->glXCreateContext = (glXCreateContextFunc_t) crDLLGetNoError( glDll, "glXCreateContext" );
	interface->glXDestroyContext = (glXDestroyContextFunc_t) crDLLGetNoError( glDll, "glXDestroyContext" );
	interface->glXUseXFont = (glXUseXFontFunc_t) crDLLGetNoError( glDll, "glXUseXFont" );
	interface->glXIsDirect = (glXIsDirectFunc_t) crDLLGetNoError( glDll, "glXIsDirect" );
	interface->glXMakeCurrent = (glXMakeCurrentFunc_t) crDLLGetNoError( glDll, "glXMakeCurrent" );
	interface->glGetString = (glGetStringFunc_t) crDLLGetNoError( glDll, "glGetString" );
	interface->glXSwapBuffers = (glXSwapBuffersFunc_t) crDLLGetNoError( glDll, "glXSwapBuffers" );
	interface->glXGetCurrentDisplay = (glXGetCurrentDisplayFunc_t) crDLLGetNoError( glDll, "glXGetCurrentDisplay" );
	interface->glXGetCurrentContext = (glXGetCurrentContextFunc_t) crDLLGetNoError( glDll, "glXGetCurrentContext" );
	interface->glXGetClientString = (glXGetClientStringFunc_t) crDLLGetNoError( glDll, "glXGetClientString" );
	interface->glXWaitGL = (glXWaitGLFunc_t) crDLLGetNoError( glDll, "glXWaitGL" );
	interface->glXWaitX = (glXWaitXFunc_t) crDLLGetNoError( glDll, "glXWaitX" );
	interface->glXCopyContext = (glXCopyContextFunc_t) crDLLGetNoError( glDll, "glXCopyContext" );
	interface->glXGetProcAddressARB = (glXGetProcAddressARBFunc_t) crDLLGetNoError( glDll, "glXGetProcAddressARB" );
	interface->glXJoinSwapGroupNV = (glXJoinSwapGroupNVFunc_t) crDLLGetNoError( glDll, "glXJoinSwapGroupNV" );
	interface->glXBindSwapBarrierNV = (glXBindSwapBarrierNVFunc_t) crDLLGetNoError( glDll, "glXBindSwapBarrierNV" );
	interface->glXQuerySwapGroupNV = (glXQuerySwapGroupNVFunc_t) crDLLGetNoError( glDll, "glXQuerySwapGroupNV" );
	interface->glXQueryMaxSwapGroupsNV = (glXQueryMaxSwapGroupsNVFunc_t) crDLLGetNoError( glDll, "glXQueryMaxSwapGroupsNV" );
	interface->glXQueryFrameCountNV = (glXQueryFrameCountNVFunc_t) crDLLGetNoError( glDll, "glXQueryFrameCountNV" );
	interface->glXResetFrameCountNV = (glXResetFrameCountNVFunc_t) crDLLGetNoError( glDll, "glXResetFrameCountNV" );
	interface->glXChooseFBConfig = (glXChooseFBConfigFunc_t) crDLLGetNoError( glDll, "glXChooseFBConfig" );
	interface->glXGetFBConfigs = (glXGetFBConfigsFunc_t) crDLLGetNoError( glDll, "glXGetFBConfigs" );
	interface->glXGetFBConfigAttrib = (glXGetFBConfigAttribFunc_t) crDLLGetNoError( glDll, "glXGetFBConfigAttrib" );
	interface->glXGetVisualFromFBConfig = (glXGetVisualFromFBConfigFunc_t) crDLLGetNoError( glDll, "glXGetVisualFromFBConfig" );
	interface->glXCreateNewContext = (glXCreateNewContextFunc_t) crDLLGetNoError( glDll, "glXCreateNewContext" );
	interface->glXCreatePbuffer = (glXCreatePbufferFunc_t) crDLLGetNoError( glDll, "glXCreatePbuffer" );
	interface->glXDestroyPbuffer = (glXDestroyPbufferFunc_t) crDLLGetNoError( glDll, "glXDestroyPbuffer" );
	interface->glXQueryContext = (glXQueryContextFunc_t) crDLLGetNoError( glDll, "glXQueryContext" );
	interface->glXQueryDrawable = (glXQueryDrawableFunc_t) crDLLGetNoError( glDll, "glXQueryDrawable" );
	interface->glXMakeContextCurrent = (glXMakeContextCurrentFunc_t) crDLLGetNoError( glDll, "glXMakeContextCurrent" );
	interface->glXCreateWindow = (glXCreateWindowFunc_t) crDLLGetNoError( glDll, "glXCreateWindow" );
	interface->glXGetVisualFromFBConfig = (glXGetVisualFromFBConfigFunc_t) crDLLGetNoError( glDll, "glXGetVisualFromFBConfig" );
#endif

	if (!entry)
		return 1; /* token value */

	for (i = 0; coreFunctions[i]; i++) {
		const char *name = coreFunctions[i];
		if (fillin(entry, name + 2, crDLLGetNoError(glDll, name)))
			entry++;
        else
            crDebug("glLoader: NULL function %s", name);
	}

	/* end of table markers */
	entry->name = NULL;
	entry->fn = NULL;
	return entry - table;  /* number of entries filled */
}


/*
 * Fill in table[] with all the OpenGL extension functions that we're
 * interested in.
 */
int
crLoadOpenGLExtensions( const crOpenGLInterface *interface, SPUNamedFunctionTable table[] )
{
	struct extfunc {
		const char *funcName;
		const char *aliasName;
		SPUGenericFunction nopFunction;
	};
	static const struct extfunc functions[] = {

		{ "glActiveStencilFaceEXT", NULL, (SPUGenericFunction) NopActiveStencilFaceEXT },
		{ "glActiveTextureARB", "glActiveTexture", (SPUGenericFunction) NopActiveTextureARB },
		{ "glAreProgramsResidentNV", NULL, (SPUGenericFunction) NopAreProgramsResidentNV },
		{ "glAttachShader", "glAttachObjectARB", (SPUGenericFunction) NopAttachShader },
		{ "glBarrierCreateCR", NULL, (SPUGenericFunction) NopBarrierCreateCR },
		{ "glBarrierDestroyCR", NULL, (SPUGenericFunction) NopBarrierDestroyCR },
		{ "glBarrierExecCR", NULL, (SPUGenericFunction) NopBarrierExecCR },
		{ "glBeginQueryARB", "glBeginQuery", (SPUGenericFunction) NopBeginQueryARB },
		{ "glBindAttribLocation", "glBindAttribLocationARB", (SPUGenericFunction) NopBindAttribLocation },
		{ "glBindBufferARB", "glBindBuffer", (SPUGenericFunction) NopBindBufferARB },
		{ "glBindFramebufferEXT", NULL, (SPUGenericFunction) NopBindFramebufferEXT },
		{ "glBindProgramARB", NULL, (SPUGenericFunction) NopBindProgramARB },
		{ "glBindProgramNV", NULL, (SPUGenericFunction) NopBindProgramNV },
		{ "glBindRenderbufferEXT", NULL, (SPUGenericFunction) NopBindRenderbufferEXT },
		{ "glBlendColorEXT", "glBlendColor", (SPUGenericFunction) NopBlendColorEXT },
		{ "glBlendEquationEXT", "glBlendEquation", (SPUGenericFunction) NopBlendEquationEXT },
		{ "glBlendEquationSeparate", "glBlendEquationSeparateEXT", (SPUGenericFunction) NopBlendEquationSeparate },
		{ "glBlendFuncSeparateEXT", "glBlendFuncSeparate", (SPUGenericFunction) NopBlendFuncSeparateEXT },
		{ "glBlitFramebufferEXT", NULL, (SPUGenericFunction) NopBlitFramebufferEXT },
		{ "glBoundsInfoCR", NULL, (SPUGenericFunction) NopBoundsInfoCR },
		{ "glBufferDataARB", "glBufferData", (SPUGenericFunction) NopBufferDataARB },
		{ "glBufferSubDataARB", "glBufferSubData", (SPUGenericFunction) NopBufferSubDataARB },
		{ "glCheckFramebufferStatusEXT", NULL, (SPUGenericFunction) NopCheckFramebufferStatusEXT },
		{ "glChromiumParameterfCR", NULL, (SPUGenericFunction) NopChromiumParameterfCR },
		{ "glChromiumParameteriCR", NULL, (SPUGenericFunction) NopChromiumParameteriCR },
		{ "glChromiumParametervCR", NULL, (SPUGenericFunction) NopChromiumParametervCR },
		{ "glClientActiveTextureARB", "glClientActiveTexture", (SPUGenericFunction) NopClientActiveTextureARB },
		{ "glCombinerInputNV", NULL, (SPUGenericFunction) NopCombinerInputNV },
		{ "glCombinerOutputNV", NULL, (SPUGenericFunction) NopCombinerOutputNV },
		{ "glCombinerParameterfNV", NULL, (SPUGenericFunction) NopCombinerParameterfNV },
		{ "glCombinerParameterfvNV", NULL, (SPUGenericFunction) NopCombinerParameterfvNV },
		{ "glCombinerParameteriNV", NULL, (SPUGenericFunction) NopCombinerParameteriNV },
		{ "glCombinerParameterivNV", NULL, (SPUGenericFunction) NopCombinerParameterivNV },
		{ "glCombinerStageParameterfvNV", NULL, (SPUGenericFunction) NopCombinerStageParameterfvNV },
		{ "glCompileShader", "glCompileShaderARB", (SPUGenericFunction) NopCompileShader },
		{ "glCompressedTexImage1DARB", "glCompressedTexImage1D", (SPUGenericFunction) NopCompressedTexImage1DARB },
		{ "glCompressedTexImage2DARB", "glCompressedTexImage2D", (SPUGenericFunction) NopCompressedTexImage2DARB },
		{ "glCompressedTexImage3DARB", "glCompressedTexImage3D", (SPUGenericFunction) NopCompressedTexImage3DARB },
		{ "glCompressedTexSubImage1DARB", "glCompressedTexSubImage1D", (SPUGenericFunction) NopCompressedTexSubImage1DARB },
		{ "glCompressedTexSubImage2DARB", "glCompressedTexSubImage2D", (SPUGenericFunction) NopCompressedTexSubImage2DARB },
		{ "glCompressedTexSubImage3DARB", "glCompressedTexSubImage3D", (SPUGenericFunction) NopCompressedTexSubImage3DARB },
		{ "crCreateContext", NULL, (SPUGenericFunction) NopCreateContext },
		{ "glCreateProgram", "glCreateProgramObjectARB", (SPUGenericFunction) NopCreateProgram },
		{ "glCreateShader", "glCreateShaderObjectARB", (SPUGenericFunction) NopCreateShader },
		{ "glDeleteBuffersARB", "glDeleteBuffers", (SPUGenericFunction) NopDeleteBuffersARB },
		{ "glDeleteFencesNV", NULL, (SPUGenericFunction) NopDeleteFencesNV },
		{ "glDeleteFramebuffersEXT", NULL, (SPUGenericFunction) NopDeleteFramebuffersEXT },
		{ "glDeleteObjectARB", NULL, (SPUGenericFunction) NopDeleteObjectARB },
		{ "glDeleteProgram", NULL, (SPUGenericFunction) NopDeleteProgram },
		{ "glDeleteProgramsARB", "glDeleteProgramsNV", (SPUGenericFunction) NopDeleteProgramsARB },
		{ "glDeleteQueriesARB", "glDeleteQueries", (SPUGenericFunction) NopDeleteQueriesARB },
		{ "glDeleteRenderbuffersEXT", NULL, (SPUGenericFunction) NopDeleteRenderbuffersEXT },
		{ "glDeleteShader", NULL, (SPUGenericFunction) NopDeleteShader },
		{ "crDestroyContext", NULL, (SPUGenericFunction) NopDestroyContext },
		{ "glDetachShader", "glDetachObjectARB", (SPUGenericFunction) NopDetachShader },
		{ "glDisableVertexAttribArrayARB", "glDisableVertexAttribArray", (SPUGenericFunction) NopDisableVertexAttribArrayARB },
		{ "glDrawBuffers", NULL, (SPUGenericFunction) NopDrawBuffers },
		{ "glEnableVertexAttribArrayARB", "glEnableVertexAttribArray", (SPUGenericFunction) NopEnableVertexAttribArrayARB },
		{ "glEndQueryARB", "glEndQuery", (SPUGenericFunction) NopEndQueryARB },
		{ "glExecuteProgramNV", NULL, (SPUGenericFunction) NopExecuteProgramNV },
		{ "glFinalCombinerInputNV", NULL, (SPUGenericFunction) NopFinalCombinerInputNV },
		{ "glFinishFenceNV", NULL, (SPUGenericFunction) NopFinishFenceNV },
		{ "glFlushVertexArrayRangeNV", NULL, (SPUGenericFunction) NopFlushVertexArrayRangeNV },
		{ "glFogCoordPointerEXT", "glFogCoordPointer", (SPUGenericFunction) NopFogCoordPointerEXT },
		{ "glFogCoorddEXT", "glFogCoordd", (SPUGenericFunction) NopFogCoorddEXT },
		{ "glFogCoorddvEXT", "glFogCoorddv", (SPUGenericFunction) NopFogCoorddvEXT },
		{ "glFogCoordfEXT", "glFogCoordf", (SPUGenericFunction) NopFogCoordfEXT },
		{ "glFogCoordfvEXT", "glFogCoordfv", (SPUGenericFunction) NopFogCoordfvEXT },
		{ "glFramebufferRenderbufferEXT", NULL, (SPUGenericFunction) NopFramebufferRenderbufferEXT },
		{ "glFramebufferTexture1DEXT", NULL, (SPUGenericFunction) NopFramebufferTexture1DEXT },
		{ "glFramebufferTexture2DEXT", NULL, (SPUGenericFunction) NopFramebufferTexture2DEXT },
		{ "glFramebufferTexture3DEXT", NULL, (SPUGenericFunction) NopFramebufferTexture3DEXT },
		{ "glGenBuffersARB", "glGenBuffers", (SPUGenericFunction) NopGenBuffersARB },
		{ "glGenFencesNV", NULL, (SPUGenericFunction) NopGenFencesNV },
		{ "glGenFramebuffersEXT", NULL, (SPUGenericFunction) NopGenFramebuffersEXT },
		{ "glGenProgramsARB", NULL, (SPUGenericFunction) NopGenProgramsARB },
		{ "glGenProgramsNV", NULL, (SPUGenericFunction) NopGenProgramsNV },
		{ "glGenQueriesARB", "glGenQueries", (SPUGenericFunction) NopGenQueriesARB },
		{ "glGenRenderbuffersEXT", NULL, (SPUGenericFunction) NopGenRenderbuffersEXT },
		{ "glGenerateMipmapEXT", NULL, (SPUGenericFunction) NopGenerateMipmapEXT },
		{ "glGetActiveAttrib", "glGetActiveAttribARB", (SPUGenericFunction) NopGetActiveAttrib },
		{ "glGetActiveUniform", "glGetActiveUniformARB", (SPUGenericFunction) NopGetActiveUniform },
		{ "glGetAttachedObjectsARB", NULL, (SPUGenericFunction) NopGetAttachedObjectsARB },
		{ "glGetAttachedShaders", NULL, (SPUGenericFunction) NopGetAttachedShaders },
		{ "glGetAttribLocation", "glGetAttribLocationARB", (SPUGenericFunction) NopGetAttribLocation },
		{ "glGetBufferParameterivARB", "glGetBufferParameteriv", (SPUGenericFunction) NopGetBufferParameterivARB },
		{ "glGetBufferPointervARB", "glGetBufferPointerv", (SPUGenericFunction) NopGetBufferPointervARB },
		{ "glGetBufferSubDataARB", "glGetBufferSubData", (SPUGenericFunction) NopGetBufferSubDataARB },
		{ "glGetChromiumParametervCR", NULL, (SPUGenericFunction) NopGetChromiumParametervCR },
		{ "glGetCombinerInputParameterfvNV", NULL, (SPUGenericFunction) NopGetCombinerInputParameterfvNV },
		{ "glGetCombinerInputParameterivNV", NULL, (SPUGenericFunction) NopGetCombinerInputParameterivNV },
		{ "glGetCombinerOutputParameterfvNV", NULL, (SPUGenericFunction) NopGetCombinerOutputParameterfvNV },
		{ "glGetCombinerOutputParameterivNV", NULL, (SPUGenericFunction) NopGetCombinerOutputParameterivNV },
		{ "glGetCombinerStageParameterfvNV", NULL, (SPUGenericFunction) NopGetCombinerStageParameterfvNV },
		{ "glGetCompressedTexImageARB", "glGetCompressedTexImage", (SPUGenericFunction) NopGetCompressedTexImageARB },
		{ "glGetFenceivNV", NULL, (SPUGenericFunction) NopGetFenceivNV },
		{ "glGetFinalCombinerInputParameterfvNV", NULL, (SPUGenericFunction) NopGetFinalCombinerInputParameterfvNV },
		{ "glGetFinalCombinerInputParameterivNV", NULL, (SPUGenericFunction) NopGetFinalCombinerInputParameterivNV },
		{ "glGetFramebufferAttachmentParameterivEXT", NULL, (SPUGenericFunction) NopGetFramebufferAttachmentParameterivEXT },
		{ "glGetHandleARB", NULL, (SPUGenericFunction) NopGetHandleARB },
		{ "glGetInfoLogARB", NULL, (SPUGenericFunction) NopGetInfoLogARB },
		{ "glGetObjectParameterfvARB", NULL, (SPUGenericFunction) NopGetObjectParameterfvARB },
		{ "glGetObjectParameterivARB", NULL, (SPUGenericFunction) NopGetObjectParameterivARB },
		{ "glGetProgramEnvParameterdvARB", NULL, (SPUGenericFunction) NopGetProgramEnvParameterdvARB },
		{ "glGetProgramEnvParameterfvARB", NULL, (SPUGenericFunction) NopGetProgramEnvParameterfvARB },
		{ "glGetProgramInfoLog", NULL, (SPUGenericFunction) NopGetProgramInfoLog },
		{ "glGetProgramLocalParameterdvARB", NULL, (SPUGenericFunction) NopGetProgramLocalParameterdvARB },
		{ "glGetProgramLocalParameterfvARB", NULL, (SPUGenericFunction) NopGetProgramLocalParameterfvARB },
		{ "glGetProgramNamedParameterdvNV", NULL, (SPUGenericFunction) NopGetProgramNamedParameterdvNV },
		{ "glGetProgramNamedParameterfvNV", NULL, (SPUGenericFunction) NopGetProgramNamedParameterfvNV },
		{ "glGetProgramParameterdvNV", NULL, (SPUGenericFunction) NopGetProgramParameterdvNV },
		{ "glGetProgramParameterfvNV", NULL, (SPUGenericFunction) NopGetProgramParameterfvNV },
		{ "glGetProgramStringARB", NULL, (SPUGenericFunction) NopGetProgramStringARB },
		{ "glGetProgramStringNV", NULL, (SPUGenericFunction) NopGetProgramStringNV },
		{ "glGetProgramiv", NULL, (SPUGenericFunction) NopGetProgramiv },
		{ "glGetProgramivARB", NULL, (SPUGenericFunction) NopGetProgramivARB },
		{ "glGetProgramivNV", NULL, (SPUGenericFunction) NopGetProgramivNV },
		{ "glGetQueryObjectivARB", "glGetQueryObjectiv", (SPUGenericFunction) NopGetQueryObjectivARB },
		{ "glGetQueryObjectuivARB", "glGetQueryObjectuiv", (SPUGenericFunction) NopGetQueryObjectuivARB },
		{ "glGetQueryivARB", "glGetQueryiv", (SPUGenericFunction) NopGetQueryivARB },
		{ "glGetRenderbufferParameterivEXT", NULL, (SPUGenericFunction) NopGetRenderbufferParameterivEXT },
		{ "glGetShaderInfoLog", NULL, (SPUGenericFunction) NopGetShaderInfoLog },
		{ "glGetShaderSource", "glGetShaderSourceARB", (SPUGenericFunction) NopGetShaderSource },
		{ "glGetShaderiv", NULL, (SPUGenericFunction) NopGetShaderiv },
		{ "glGetTrackMatrixivNV", NULL, (SPUGenericFunction) NopGetTrackMatrixivNV },
		{ "glGetUniformLocation", "glGetUniformLocationARB", (SPUGenericFunction) NopGetUniformLocation },
		{ "glGetUniformfv", "glGetUniformfvARB", (SPUGenericFunction) NopGetUniformfv },
		{ "glGetUniformiv", "glGetUniformivARB", (SPUGenericFunction) NopGetUniformiv },
		{ "crGetUniformsLocations", NULL, (SPUGenericFunction) NopGetUniformsLocations },
		{ "glGetVertexAttribPointervARB", "glGetVertexAttribPointerv", (SPUGenericFunction) NopGetVertexAttribPointervARB },
		{ "glGetVertexAttribPointervNV", NULL, (SPUGenericFunction) NopGetVertexAttribPointervNV },
		{ "glGetVertexAttribdvARB", "glGetVertexAttribdv", (SPUGenericFunction) NopGetVertexAttribdvARB },
		{ "glGetVertexAttribdvNV", NULL, (SPUGenericFunction) NopGetVertexAttribdvNV },
		{ "glGetVertexAttribfvARB", "glGetVertexAttribfv", (SPUGenericFunction) NopGetVertexAttribfvARB },
		{ "glGetVertexAttribfvNV", NULL, (SPUGenericFunction) NopGetVertexAttribfvNV },
		{ "glGetVertexAttribivARB", "glGetVertexAttribiv", (SPUGenericFunction) NopGetVertexAttribivARB },
		{ "glGetVertexAttribivNV", NULL, (SPUGenericFunction) NopGetVertexAttribivNV },
		{ "glIsBufferARB", "glIsBuffer", (SPUGenericFunction) NopIsBufferARB },
		{ "glIsFenceNV", NULL, (SPUGenericFunction) NopIsFenceNV },
		{ "glIsFramebufferEXT", NULL, (SPUGenericFunction) NopIsFramebufferEXT },
		{ "glIsProgram", NULL, (SPUGenericFunction) NopIsProgram },
		{ "glIsProgramARB", "glIsProgramNV", (SPUGenericFunction) NopIsProgramARB },
		{ "glIsQueryARB", "glIsQuery", (SPUGenericFunction) NopIsQueryARB },
		{ "glIsRenderbufferEXT", NULL, (SPUGenericFunction) NopIsRenderbufferEXT },
		{ "glIsShader", NULL, (SPUGenericFunction) NopIsShader },
		{ "glLinkProgram", "glLinkProgramARB", (SPUGenericFunction) NopLinkProgram },
		{ "glLoadProgramNV", NULL, (SPUGenericFunction) NopLoadProgramNV },
		{ "glLoadTransposeMatrixdARB", "glLoadTransposeMatrixd", (SPUGenericFunction) NopLoadTransposeMatrixdARB },
		{ "glLoadTransposeMatrixfARB", "glLoadTransposeMatrixf", (SPUGenericFunction) NopLoadTransposeMatrixfARB },
		{ "glLockArraysEXT", NULL, (SPUGenericFunction) NopLockArraysEXT },
		{ "crMakeCurrent", NULL, (SPUGenericFunction) NopMakeCurrent },
		{ "glMapBufferARB", "glMapBuffer", (SPUGenericFunction) NopMapBufferARB },
		{ "glMultTransposeMatrixdARB", "glMultTransposeMatrixd", (SPUGenericFunction) NopMultTransposeMatrixdARB },
		{ "glMultTransposeMatrixfARB", "glMultTransposeMatrixf", (SPUGenericFunction) NopMultTransposeMatrixfARB },
		{ "glMultiDrawArraysEXT", "glMultiDrawArrays", (SPUGenericFunction) NopMultiDrawArraysEXT },
		{ "glMultiDrawElementsEXT", "glMultiDrawElements", (SPUGenericFunction) NopMultiDrawElementsEXT },
		{ "glMultiTexCoord1dARB", "glMultiTexCoord1d", (SPUGenericFunction) NopMultiTexCoord1dARB },
		{ "glMultiTexCoord1dvARB", "glMultiTexCoord1dv", (SPUGenericFunction) NopMultiTexCoord1dvARB },
		{ "glMultiTexCoord1fARB", "glMultiTexCoord1f", (SPUGenericFunction) NopMultiTexCoord1fARB },
		{ "glMultiTexCoord1fvARB", "glMultiTexCoord1fv", (SPUGenericFunction) NopMultiTexCoord1fvARB },
		{ "glMultiTexCoord1iARB", "glMultiTexCoord1i", (SPUGenericFunction) NopMultiTexCoord1iARB },
		{ "glMultiTexCoord1ivARB", "glMultiTexCoord1iv", (SPUGenericFunction) NopMultiTexCoord1ivARB },
		{ "glMultiTexCoord1sARB", "glMultiTexCoord1s", (SPUGenericFunction) NopMultiTexCoord1sARB },
		{ "glMultiTexCoord1svARB", "glMultiTexCoord1sv", (SPUGenericFunction) NopMultiTexCoord1svARB },
		{ "glMultiTexCoord2dARB", "glMultiTexCoord2d", (SPUGenericFunction) NopMultiTexCoord2dARB },
		{ "glMultiTexCoord2dvARB", "glMultiTexCoord2dv", (SPUGenericFunction) NopMultiTexCoord2dvARB },
		{ "glMultiTexCoord2fARB", "glMultiTexCoord2f", (SPUGenericFunction) NopMultiTexCoord2fARB },
		{ "glMultiTexCoord2fvARB", "glMultiTexCoord2fv", (SPUGenericFunction) NopMultiTexCoord2fvARB },
		{ "glMultiTexCoord2iARB", "glMultiTexCoord2i", (SPUGenericFunction) NopMultiTexCoord2iARB },
		{ "glMultiTexCoord2ivARB", "glMultiTexCoord2iv", (SPUGenericFunction) NopMultiTexCoord2ivARB },
		{ "glMultiTexCoord2sARB", "glMultiTexCoord2s", (SPUGenericFunction) NopMultiTexCoord2sARB },
		{ "glMultiTexCoord2svARB", "glMultiTexCoord2sv", (SPUGenericFunction) NopMultiTexCoord2svARB },
		{ "glMultiTexCoord3dARB", "glMultiTexCoord3d", (SPUGenericFunction) NopMultiTexCoord3dARB },
		{ "glMultiTexCoord3dvARB", "glMultiTexCoord3dv", (SPUGenericFunction) NopMultiTexCoord3dvARB },
		{ "glMultiTexCoord3fARB", "glMultiTexCoord3f", (SPUGenericFunction) NopMultiTexCoord3fARB },
		{ "glMultiTexCoord3fvARB", "glMultiTexCoord3fv", (SPUGenericFunction) NopMultiTexCoord3fvARB },
		{ "glMultiTexCoord3iARB", "glMultiTexCoord3i", (SPUGenericFunction) NopMultiTexCoord3iARB },
		{ "glMultiTexCoord3ivARB", "glMultiTexCoord3iv", (SPUGenericFunction) NopMultiTexCoord3ivARB },
		{ "glMultiTexCoord3sARB", "glMultiTexCoord3s", (SPUGenericFunction) NopMultiTexCoord3sARB },
		{ "glMultiTexCoord3svARB", "glMultiTexCoord3sv", (SPUGenericFunction) NopMultiTexCoord3svARB },
		{ "glMultiTexCoord4dARB", "glMultiTexCoord4d", (SPUGenericFunction) NopMultiTexCoord4dARB },
		{ "glMultiTexCoord4dvARB", "glMultiTexCoord4dv", (SPUGenericFunction) NopMultiTexCoord4dvARB },
		{ "glMultiTexCoord4fARB", "glMultiTexCoord4f", (SPUGenericFunction) NopMultiTexCoord4fARB },
		{ "glMultiTexCoord4fvARB", "glMultiTexCoord4fv", (SPUGenericFunction) NopMultiTexCoord4fvARB },
		{ "glMultiTexCoord4iARB", "glMultiTexCoord4i", (SPUGenericFunction) NopMultiTexCoord4iARB },
		{ "glMultiTexCoord4ivARB", "glMultiTexCoord4iv", (SPUGenericFunction) NopMultiTexCoord4ivARB },
		{ "glMultiTexCoord4sARB", "glMultiTexCoord4s", (SPUGenericFunction) NopMultiTexCoord4sARB },
		{ "glMultiTexCoord4svARB", "glMultiTexCoord4sv", (SPUGenericFunction) NopMultiTexCoord4svARB },
		{ "glPointParameterfARB", "glPointParameterf", (SPUGenericFunction) NopPointParameterfARB },
		{ "glPointParameterfvARB", "glPointParameterfv", (SPUGenericFunction) NopPointParameterfvARB },
		{ "glPointParameteri", "glPointParameteriNV", (SPUGenericFunction) NopPointParameteri },
		{ "glPointParameteriv", "glPointParameterivNV", (SPUGenericFunction) NopPointParameteriv },
		{ "glProgramEnvParameter4dARB", NULL, (SPUGenericFunction) NopProgramEnvParameter4dARB },
		{ "glProgramEnvParameter4dvARB", NULL, (SPUGenericFunction) NopProgramEnvParameter4dvARB },
		{ "glProgramEnvParameter4fARB", NULL, (SPUGenericFunction) NopProgramEnvParameter4fARB },
		{ "glProgramEnvParameter4fvARB", NULL, (SPUGenericFunction) NopProgramEnvParameter4fvARB },
		{ "glProgramLocalParameter4dARB", NULL, (SPUGenericFunction) NopProgramLocalParameter4dARB },
		{ "glProgramLocalParameter4dvARB", NULL, (SPUGenericFunction) NopProgramLocalParameter4dvARB },
		{ "glProgramLocalParameter4fARB", NULL, (SPUGenericFunction) NopProgramLocalParameter4fARB },
		{ "glProgramLocalParameter4fvARB", NULL, (SPUGenericFunction) NopProgramLocalParameter4fvARB },
		{ "glProgramNamedParameter4dNV", NULL, (SPUGenericFunction) NopProgramNamedParameter4dNV },
		{ "glProgramNamedParameter4dvNV", NULL, (SPUGenericFunction) NopProgramNamedParameter4dvNV },
		{ "glProgramNamedParameter4fNV", NULL, (SPUGenericFunction) NopProgramNamedParameter4fNV },
		{ "glProgramNamedParameter4fvNV", NULL, (SPUGenericFunction) NopProgramNamedParameter4fvNV },
		{ "glProgramParameter4dNV", NULL, (SPUGenericFunction) NopProgramParameter4dNV },
		{ "glProgramParameter4dvNV", NULL, (SPUGenericFunction) NopProgramParameter4dvNV },
		{ "glProgramParameter4fNV", NULL, (SPUGenericFunction) NopProgramParameter4fNV },
		{ "glProgramParameter4fvNV", NULL, (SPUGenericFunction) NopProgramParameter4fvNV },
		{ "glProgramParameters4dvNV", NULL, (SPUGenericFunction) NopProgramParameters4dvNV },
		{ "glProgramParameters4fvNV", NULL, (SPUGenericFunction) NopProgramParameters4fvNV },
		{ "glProgramStringARB", NULL, (SPUGenericFunction) NopProgramStringARB },
		{ "glRenderbufferStorageEXT", NULL, (SPUGenericFunction) NopRenderbufferStorageEXT },
		{ "glRequestResidentProgramsNV", NULL, (SPUGenericFunction) NopRequestResidentProgramsNV },
		{ "glSampleCoverageARB", "glSampleCoverage", (SPUGenericFunction) NopSampleCoverageARB },
		{ "glSecondaryColor3bEXT", "glSecondaryColor3b", (SPUGenericFunction) NopSecondaryColor3bEXT },
		{ "glSecondaryColor3bvEXT", "glSecondaryColor3bv", (SPUGenericFunction) NopSecondaryColor3bvEXT },
		{ "glSecondaryColor3dEXT", "glSecondaryColor3d", (SPUGenericFunction) NopSecondaryColor3dEXT },
		{ "glSecondaryColor3dvEXT", "glSecondaryColor3dv", (SPUGenericFunction) NopSecondaryColor3dvEXT },
		{ "glSecondaryColor3fEXT", "glSecondaryColor3f", (SPUGenericFunction) NopSecondaryColor3fEXT },
		{ "glSecondaryColor3fvEXT", "glSecondaryColor3fv", (SPUGenericFunction) NopSecondaryColor3fvEXT },
		{ "glSecondaryColor3iEXT", "glSecondaryColor3i", (SPUGenericFunction) NopSecondaryColor3iEXT },
		{ "glSecondaryColor3ivEXT", "glSecondaryColor3iv", (SPUGenericFunction) NopSecondaryColor3ivEXT },
		{ "glSecondaryColor3sEXT", "glSecondaryColor3s", (SPUGenericFunction) NopSecondaryColor3sEXT },
		{ "glSecondaryColor3svEXT", "glSecondaryColor3sv", (SPUGenericFunction) NopSecondaryColor3svEXT },
		{ "glSecondaryColor3ubEXT", "glSecondaryColor3ub", (SPUGenericFunction) NopSecondaryColor3ubEXT },
		{ "glSecondaryColor3ubvEXT", "glSecondaryColor3ubv", (SPUGenericFunction) NopSecondaryColor3ubvEXT },
		{ "glSecondaryColor3uiEXT", "glSecondaryColor3ui", (SPUGenericFunction) NopSecondaryColor3uiEXT },
		{ "glSecondaryColor3uivEXT", "glSecondaryColor3uiv", (SPUGenericFunction) NopSecondaryColor3uivEXT },
		{ "glSecondaryColor3usEXT", "glSecondaryColor3us", (SPUGenericFunction) NopSecondaryColor3usEXT },
		{ "glSecondaryColor3usvEXT", "glSecondaryColor3usv", (SPUGenericFunction) NopSecondaryColor3usvEXT },
		{ "glSecondaryColorPointerEXT", "glSecondaryColorPointer", (SPUGenericFunction) NopSecondaryColorPointerEXT },
		{ "glSemaphoreCreateCR", NULL, (SPUGenericFunction) NopSemaphoreCreateCR },
		{ "glSemaphoreDestroyCR", NULL, (SPUGenericFunction) NopSemaphoreDestroyCR },
		{ "glSemaphorePCR", NULL, (SPUGenericFunction) NopSemaphorePCR },
		{ "glSemaphoreVCR", NULL, (SPUGenericFunction) NopSemaphoreVCR },
		{ "glSetFenceNV", NULL, (SPUGenericFunction) NopSetFenceNV },
		{ "glShaderSource", "glShaderSourceARB", (SPUGenericFunction) NopShaderSource },
		{ "glStencilFuncSeparate", NULL, (SPUGenericFunction) NopStencilFuncSeparate },
		{ "glStencilMaskSeparate", NULL, (SPUGenericFunction) NopStencilMaskSeparate },
		{ "glStencilOpSeparate", NULL, (SPUGenericFunction) NopStencilOpSeparate },
		{ "crSwapBuffers", NULL, (SPUGenericFunction) NopSwapBuffers },
		{ "glTestFenceNV", NULL, (SPUGenericFunction) NopTestFenceNV },
		{ "glTexImage3DEXT", NULL, (SPUGenericFunction) NopTexImage3DEXT },
		{ "glTrackMatrixNV", NULL, (SPUGenericFunction) NopTrackMatrixNV },
		{ "glUniform1f", "glUniform1fARB", (SPUGenericFunction) NopUniform1f },
		{ "glUniform1fv", "glUniform1fvARB", (SPUGenericFunction) NopUniform1fv },
		{ "glUniform1i", "glUniform1iARB", (SPUGenericFunction) NopUniform1i },
		{ "glUniform1iv", "glUniform1ivARB", (SPUGenericFunction) NopUniform1iv },
		{ "glUniform2f", "glUniform2fARB", (SPUGenericFunction) NopUniform2f },
		{ "glUniform2fv", "glUniform2fvARB", (SPUGenericFunction) NopUniform2fv },
		{ "glUniform2i", "glUniform2iARB", (SPUGenericFunction) NopUniform2i },
		{ "glUniform2iv", "glUniform2ivARB", (SPUGenericFunction) NopUniform2iv },
		{ "glUniform3f", "glUniform3fARB", (SPUGenericFunction) NopUniform3f },
		{ "glUniform3fv", "glUniform3fvARB", (SPUGenericFunction) NopUniform3fv },
		{ "glUniform3i", "glUniform3iARB", (SPUGenericFunction) NopUniform3i },
		{ "glUniform3iv", "glUniform3ivARB", (SPUGenericFunction) NopUniform3iv },
		{ "glUniform4f", "glUniform4fARB", (SPUGenericFunction) NopUniform4f },
		{ "glUniform4fv", "glUniform4fvARB", (SPUGenericFunction) NopUniform4fv },
		{ "glUniform4i", "glUniform4iARB", (SPUGenericFunction) NopUniform4i },
		{ "glUniform4iv", "glUniform4ivARB", (SPUGenericFunction) NopUniform4iv },
		{ "glUniformMatrix2fv", "glUniformMatrix2fvARB", (SPUGenericFunction) NopUniformMatrix2fv },
		{ "glUniformMatrix2x3fv", NULL, (SPUGenericFunction) NopUniformMatrix2x3fv },
		{ "glUniformMatrix2x4fv", NULL, (SPUGenericFunction) NopUniformMatrix2x4fv },
		{ "glUniformMatrix3fv", "glUniformMatrix3fvARB", (SPUGenericFunction) NopUniformMatrix3fv },
		{ "glUniformMatrix3x2fv", NULL, (SPUGenericFunction) NopUniformMatrix3x2fv },
		{ "glUniformMatrix3x4fv", NULL, (SPUGenericFunction) NopUniformMatrix3x4fv },
		{ "glUniformMatrix4fv", "glUniformMatrix4fvARB", (SPUGenericFunction) NopUniformMatrix4fv },
		{ "glUniformMatrix4x2fv", NULL, (SPUGenericFunction) NopUniformMatrix4x2fv },
		{ "glUniformMatrix4x3fv", NULL, (SPUGenericFunction) NopUniformMatrix4x3fv },
		{ "glUnlockArraysEXT", NULL, (SPUGenericFunction) NopUnlockArraysEXT },
		{ "glUnmapBufferARB", "glUnmapBuffer", (SPUGenericFunction) NopUnmapBufferARB },
		{ "glUseProgram", "glUseProgramObjectARB", (SPUGenericFunction) NopUseProgram },
		{ "glVBoxAttachThread", NULL, (SPUGenericFunction) NopVBoxAttachThread },
		{ "glVBoxConCreate", NULL, (SPUGenericFunction) NopVBoxConCreate },
		{ "glVBoxConDestroy", NULL, (SPUGenericFunction) NopVBoxConDestroy },
		{ "glVBoxConFlush", NULL, (SPUGenericFunction) NopVBoxConFlush },
		{ "glVBoxCreateContext", NULL, (SPUGenericFunction) NopVBoxCreateContext },
		{ "glVBoxDetachThread", NULL, (SPUGenericFunction) NopVBoxDetachThread },
		{ "glVBoxPackGetInjectID", NULL, (SPUGenericFunction) NopVBoxPackGetInjectID },
		{ "glVBoxPackSetInjectID", NULL, (SPUGenericFunction) NopVBoxPackSetInjectID },
		{ "glVBoxPackSetInjectThread", NULL, (SPUGenericFunction) NopVBoxPackSetInjectThread },
		{ "glVBoxWindowCreate", NULL, (SPUGenericFunction) NopVBoxWindowCreate },
		{ "glVBoxWindowDestroy", NULL, (SPUGenericFunction) NopVBoxWindowDestroy },
		{ "glValidateProgram", "glValidateProgramARB", (SPUGenericFunction) NopValidateProgram },
		{ "glVertexArrayRangeNV", NULL, (SPUGenericFunction) NopVertexArrayRangeNV },
		{ "glVertexAttrib1dARB", "glVertexAttrib1dNV", (SPUGenericFunction) NopVertexAttrib1dARB },
		{ "glVertexAttrib1dvARB", "glVertexAttrib1dvNV", (SPUGenericFunction) NopVertexAttrib1dvARB },
		{ "glVertexAttrib1fARB", "glVertexAttrib1f", (SPUGenericFunction) NopVertexAttrib1fARB },
		{ "glVertexAttrib1fvARB", "glVertexAttrib1fv", (SPUGenericFunction) NopVertexAttrib1fvARB },
		{ "glVertexAttrib1sARB", "glVertexAttrib1sNV", (SPUGenericFunction) NopVertexAttrib1sARB },
		{ "glVertexAttrib1svARB", "glVertexAttrib1svNV", (SPUGenericFunction) NopVertexAttrib1svARB },
		{ "glVertexAttrib2dARB", "glVertexAttrib2d", (SPUGenericFunction) NopVertexAttrib2dARB },
		{ "glVertexAttrib2dvARB", "glVertexAttrib2dvNV", (SPUGenericFunction) NopVertexAttrib2dvARB },
		{ "glVertexAttrib2fARB", "glVertexAttrib2fNV", (SPUGenericFunction) NopVertexAttrib2fARB },
		{ "glVertexAttrib2fvARB", "glVertexAttrib2fv", (SPUGenericFunction) NopVertexAttrib2fvARB },
		{ "glVertexAttrib2sARB", "glVertexAttrib2s", (SPUGenericFunction) NopVertexAttrib2sARB },
		{ "glVertexAttrib2svARB", "glVertexAttrib2svNV", (SPUGenericFunction) NopVertexAttrib2svARB },
		{ "glVertexAttrib3dARB", "glVertexAttrib3dNV", (SPUGenericFunction) NopVertexAttrib3dARB },
		{ "glVertexAttrib3dvARB", "glVertexAttrib3dvNV", (SPUGenericFunction) NopVertexAttrib3dvARB },
		{ "glVertexAttrib3fARB", "glVertexAttrib3fNV", (SPUGenericFunction) NopVertexAttrib3fARB },
		{ "glVertexAttrib3fvARB", "glVertexAttrib3fvNV", (SPUGenericFunction) NopVertexAttrib3fvARB },
		{ "glVertexAttrib3sARB", "glVertexAttrib3sNV", (SPUGenericFunction) NopVertexAttrib3sARB },
		{ "glVertexAttrib3svARB", "glVertexAttrib3svNV", (SPUGenericFunction) NopVertexAttrib3svARB },
		{ "glVertexAttrib4NbvARB", "glVertexAttrib4Nbv", (SPUGenericFunction) NopVertexAttrib4NbvARB },
		{ "glVertexAttrib4NivARB", "glVertexAttrib4Niv", (SPUGenericFunction) NopVertexAttrib4NivARB },
		{ "glVertexAttrib4NsvARB", "glVertexAttrib4Nsv", (SPUGenericFunction) NopVertexAttrib4NsvARB },
		{ "glVertexAttrib4NubARB", "glVertexAttrib4ubNV", (SPUGenericFunction) NopVertexAttrib4NubARB },
		{ "glVertexAttrib4NubvARB", "glVertexAttrib4Nubv", (SPUGenericFunction) NopVertexAttrib4NubvARB },
		{ "glVertexAttrib4NuivARB", "glVertexAttrib4Nuiv", (SPUGenericFunction) NopVertexAttrib4NuivARB },
		{ "glVertexAttrib4NusvARB", "glVertexAttrib4Nusv", (SPUGenericFunction) NopVertexAttrib4NusvARB },
		{ "glVertexAttrib4bvARB", "glVertexAttrib4bv", (SPUGenericFunction) NopVertexAttrib4bvARB },
		{ "glVertexAttrib4dARB", "glVertexAttrib4dNV", (SPUGenericFunction) NopVertexAttrib4dARB },
		{ "glVertexAttrib4dvARB", "glVertexAttrib4dvNV", (SPUGenericFunction) NopVertexAttrib4dvARB },
		{ "glVertexAttrib4fARB", "glVertexAttrib4f", (SPUGenericFunction) NopVertexAttrib4fARB },
		{ "glVertexAttrib4fvARB", "glVertexAttrib4fvNV", (SPUGenericFunction) NopVertexAttrib4fvARB },
		{ "glVertexAttrib4ivARB", "glVertexAttrib4iv", (SPUGenericFunction) NopVertexAttrib4ivARB },
		{ "glVertexAttrib4sARB", "glVertexAttrib4s", (SPUGenericFunction) NopVertexAttrib4sARB },
		{ "glVertexAttrib4svARB", "glVertexAttrib4sv", (SPUGenericFunction) NopVertexAttrib4svARB },
		{ "glVertexAttrib4ubvARB", "glVertexAttrib4ubv", (SPUGenericFunction) NopVertexAttrib4ubvARB },
		{ "glVertexAttrib4uivARB", "glVertexAttrib4uiv", (SPUGenericFunction) NopVertexAttrib4uivARB },
		{ "glVertexAttrib4usvARB", "glVertexAttrib4usv", (SPUGenericFunction) NopVertexAttrib4usvARB },
		{ "glVertexAttribPointerARB", "glVertexAttribPointer", (SPUGenericFunction) NopVertexAttribPointerARB },
		{ "glVertexAttribPointerNV", NULL, (SPUGenericFunction) NopVertexAttribPointerNV },
		{ "glVertexAttribs1dvNV", NULL, (SPUGenericFunction) NopVertexAttribs1dvNV },
		{ "glVertexAttribs1fvNV", NULL, (SPUGenericFunction) NopVertexAttribs1fvNV },
		{ "glVertexAttribs1svNV", NULL, (SPUGenericFunction) NopVertexAttribs1svNV },
		{ "glVertexAttribs2dvNV", NULL, (SPUGenericFunction) NopVertexAttribs2dvNV },
		{ "glVertexAttribs2fvNV", NULL, (SPUGenericFunction) NopVertexAttribs2fvNV },
		{ "glVertexAttribs2svNV", NULL, (SPUGenericFunction) NopVertexAttribs2svNV },
		{ "glVertexAttribs3dvNV", NULL, (SPUGenericFunction) NopVertexAttribs3dvNV },
		{ "glVertexAttribs3fvNV", NULL, (SPUGenericFunction) NopVertexAttribs3fvNV },
		{ "glVertexAttribs3svNV", NULL, (SPUGenericFunction) NopVertexAttribs3svNV },
		{ "glVertexAttribs4dvNV", NULL, (SPUGenericFunction) NopVertexAttribs4dvNV },
		{ "glVertexAttribs4fvNV", NULL, (SPUGenericFunction) NopVertexAttribs4fvNV },
		{ "glVertexAttribs4svNV", NULL, (SPUGenericFunction) NopVertexAttribs4svNV },
		{ "glVertexAttribs4ubvNV", NULL, (SPUGenericFunction) NopVertexAttribs4ubvNV },
		{ "crWindowCreate", NULL, (SPUGenericFunction) NopWindowCreate },
		{ "crWindowDestroy", NULL, (SPUGenericFunction) NopWindowDestroy },
		{ "glWindowPos2dARB", "glWindowPos2d", (SPUGenericFunction) NopWindowPos2dARB },
		{ "glWindowPos2dvARB", "glWindowPos2dv", (SPUGenericFunction) NopWindowPos2dvARB },
		{ "glWindowPos2fARB", "glWindowPos2f", (SPUGenericFunction) NopWindowPos2fARB },
		{ "glWindowPos2fvARB", "glWindowPos2fv", (SPUGenericFunction) NopWindowPos2fvARB },
		{ "glWindowPos2iARB", "glWindowPos2i", (SPUGenericFunction) NopWindowPos2iARB },
		{ "glWindowPos2ivARB", "glWindowPos2iv", (SPUGenericFunction) NopWindowPos2ivARB },
		{ "glWindowPos2sARB", "glWindowPos2s", (SPUGenericFunction) NopWindowPos2sARB },
		{ "glWindowPos2svARB", "glWindowPos2sv", (SPUGenericFunction) NopWindowPos2svARB },
		{ "glWindowPos3dARB", "glWindowPos3d", (SPUGenericFunction) NopWindowPos3dARB },
		{ "glWindowPos3dvARB", "glWindowPos3dv", (SPUGenericFunction) NopWindowPos3dvARB },
		{ "glWindowPos3fARB", "glWindowPos3f", (SPUGenericFunction) NopWindowPos3fARB },
		{ "glWindowPos3fvARB", "glWindowPos3fv", (SPUGenericFunction) NopWindowPos3fvARB },
		{ "glWindowPos3iARB", "glWindowPos3i", (SPUGenericFunction) NopWindowPos3iARB },
		{ "glWindowPos3ivARB", "glWindowPos3iv", (SPUGenericFunction) NopWindowPos3ivARB },
		{ "glWindowPos3sARB", "glWindowPos3s", (SPUGenericFunction) NopWindowPos3sARB },
		{ "glWindowPos3svARB", "glWindowPos3sv", (SPUGenericFunction) NopWindowPos3svARB },
		{ "crWindowPosition", NULL, (SPUGenericFunction) NopWindowPosition },
		{ "crWindowShow", NULL, (SPUGenericFunction) NopWindowShow },
		{ "crWindowSize", NULL, (SPUGenericFunction) NopWindowSize },
		{ "crWindowVisibleRegion", NULL, (SPUGenericFunction) NopWindowVisibleRegion },
		{ "crWriteback", NULL, (SPUGenericFunction) NopWriteback },
		{ "glZPixCR", NULL, (SPUGenericFunction) NopZPixCR },

		{ NULL, NULL, NULL}
	};
	const struct extfunc *func;
	SPUNamedFunctionTable *entry = table;

#ifdef WINDOWS
	if (interface->wglGetProcAddress == NULL)
		crWarning("Unable to find wglGetProcAddress() in system GL library");
#elif !defined(DARWIN)
	if (interface->glXGetProcAddressARB == NULL)
		crWarning("Unable to find glXGetProcAddressARB() in system GL library");
#endif

	for (func = functions; func->funcName; func++) {
		SPUGenericFunction f = findExtFunction(interface, func->funcName);
		if (!f && func->aliasName) {
			f = findExtFunction(interface, func->aliasName);
		}
		if (!f) {
			f = func->nopFunction;
		}
		(void) fillin(entry, func->funcName + 2 , f);  /* +2 to skip "gl" */
		entry++;
	}

	/* end of list */
	entry->name = NULL;
	entry->fn = NULL;
	return entry - table;  /* number of entries filled */
}



#ifdef USE_OSMESA
int crLoadOSMesa( OSMesaContext (**createContext)( GLenum format, OSMesaContext sharelist ),
		     GLboolean (**makeCurrent)( OSMesaContext ctx, GLubyte *buffer,
						GLenum type, GLsizei width, GLsizei height ),
		     void (**destroyContext)( OSMesaContext ctx ))
{
	static CRDLL *osMesaDll = NULL;

	const char *env_syspath = crGetenv( "CR_SYSTEM_GL_PATH" );

	crDebug( "Looking for the system's OSMesa library..." );
	osMesaDll = __findSystemLib( env_syspath, "libOSMesa.so" );
	if (!osMesaDll)
	{
		crError("Unable to find system OSMesa!");
		return 0;
	}

	crDebug( "Found it in %s.", !env_syspath ? "default path" : env_syspath );

	*createContext =  (OSMesaContext (*) ( GLenum format, OSMesaContext sharelist ))
		crDLLGetNoError( osMesaDll, "OSMesaCreateContext" );

	*makeCurrent =  (GLboolean (*) ( OSMesaContext ctx, GLubyte *buffer,
					  GLenum type, GLsizei width, GLsizei height ))
		crDLLGetNoError( osMesaDll, "OSMesaMakeCurrent" );

	*destroyContext =  (void (*) ( OSMesaContext ctx))
		crDLLGetNoError( osMesaDll, "OSMesaDestroyContext" );

	return 1;
}
#endif


